C************************************************************************
C************************************************************************
C*                                                                     **
C*           IPAC-NOMINATE                                             **
C*           ONE-DIMENSIONAL NOMINAL THREE-STEP ESTIMATION             **
C*                           ******  *          *                      **
C*           DEVELOPED BY NOLAN M. McCARTY, KEITH T. POOLE             **
C*                        AND HOWARD ROSENTHAL                         **
C*           CARNEGIE-MELLON UNIVERSITY, PITTSBURGH, PA.               **
C*                                                                     **
C*           INITIAL PROGRAM DEVELOPED 1982-1984                       **
C*           THIS VERSION CURRENT AS OF FEBRUARY 19, 1991              **
C************************************************************************
C************************************************************************
C
C     THE PARAMETERS BEING ESTIMATED ARE: A SIGNAL TO NOISE RATIO CONSTANT   
C     IN UTILITY FUNCTION PHASE. PAC COORDINATES (IN PAC PHASE).
C     MIDPOINT AND DISTANCE BETWEEN THE TWO OUTCOMES IN THE CONTEST PHASE. 
C     ALL PARAMETERS ARE HELD FIXED EXCEPT DURING THEIR RESPECTIVE PHASES.
C     DURING UTILITY PHASE, PAC AND CONTEST PARAMETERS ARE HELD FIXED
C     AND ARE STORED IN XDATA(.) AND ZMID(.)/DYN(.) RESPECTIVELY.  DURING
C     CONTEST PHASE, THE UTILITY FUNCTION CONSTANT, STORED IN BBB(1), IS
C     HELD FIXED AND THE PACS ARE HELD FIXED.  DURING THE PAC PHASE,
C     UTILITY AND CONTEST PARAMETERS ARE HELD FIXED.
C
C     THE FOLLOWING TABLE SHOWS WHAT IS STORED WHERE AND WHAT IS HELD CONSTANT
C     DURING EACH PHASE.
C
C	NOBS=# OF PACS, NRCALL=# OF CONTEST.
C
C                     UTILITY PHASE         PAC PHASE     CONTEST PHASE
C
C     BETA (SIG/NOISE)   B(1)               BBB(1)             BBB(1)
C     
C        PAC             XDATA(1) TO        B(1)               XDATA(1) TO
C       COORDS           XDATA(NOBS)                           XDATA(NOBS)
C
C    UTILITY THRESH.     UCUT(1) TO         B(2)              UCUT(1)
C                          UCUT(NOBS)                       UCUT(NOBS)
C
C     INCUBENT ADV.       UINC(1) TO         B(3)              UINC(1)
C                         UINC(NOBS)                          UINC(NOBS)
C
C     DISTANCE BET.      DYN(1) TO          DYN(1) TO          B(1)
C       PLATFORMS       DYN(NRCALL)        DYN(NRCALL)
C
C     MID PTS.           ZMID(1) TO         ZMID(1) TO         B(2)
C                        ZMID(NRCALL)       ZMID(NRCALL)
C
C
C     IN THE CONTEST PHASE, SUMMING TAKES PLACE OVER THE PACS  
C     AND ONLY THE PARAMETERS OF ONE CONTEST AT A TIME ARE UPDATED 
C     AND THESE ARE HELD IN B(1) AND B(2).  LIKEWISE, IN THE PAC
C     PHASE SUMMING TAKES PLACE OVER THE CONTESTS AND ONLY ONE SET OF PAC
C     COORDINATES (B(1),B(2),B(3)) IS UPDATED AT A TIME.  IN THE UTILITY 
C     PHASE SUMMATION IS OVER ALL THE DATA AND THE ESTIMATE OF BETA 
C     (THE SIGNAL TO NOISE RATIO) IS HELD IN B(1).     
C
C
      DIMENSION ALP(4),BTA(4),RR(4),FMT(20),KDATA(1601),
     CLL(4,4),LMO(513)
      character*1 kname
      CHARACTER*64 TITLE
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /LUCK/ YBIGL(1601),YYBIGL(1601)
      COMMON /ROLL/ KPJP(1601,10)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),xinc(1601),rc(500)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /SAVE/ XSAVE(513,8,2),ZSAVE(1601,8,2),CSAVE(1601,8),
     CQSAVE(1601,8,2),RSAVE(1601,8),SSAVE(513,8,2)
      COMMON /TCHOIC/ KPTSU
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      COMMON /TOL/ TOLB
      COMMON /BOOK/ BBB(50)
      COMMON /VOTES/ KAV(1601),kay(1601),kan(1601),kname(513,25),jne
      COMMON/ INTGER /IIII
      COMMON /BAD/KBAD1,KBAD2,SPRM,SPRSD
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
      OPEN(3,FILE='Mat81i.DAT',STATUS='OLD')
      OPEN(4,FILE='PAC77.DAT',STATUS='OLD')
      open(25,file='CORd81i4.DAT',status='old')
      OPEN(26,FILE='INFO77.DAT')
      OPEN(21,FILE='PAC21.NVM')
      OPEN(23,FILE='PAC23.NVM')
      OPEN(31,FILE='PAC31.NVM')
      OPEN(33,FILE='PAC33.NVM')
      DATA ISENS/1000*1/
  100 FORMAT(2I5,10F10.4)
  101 FORMAT(10X,10F10.4)
 1000 format(22x,4(i2.2,'.'))
 1001 FORMAT(8I5)
 1002 FORMAT(30X,3F14.4)
 1003 FORMAT(15F8.4)
 1004 FORMAT(20A4)
 1005 FORMAT(1X,19A4)
 1006 FORMAT(F8.4,I5)
 1007 FORMAT(3A4)
 1008 FORMAT(' DATA     ',I6,I7)
 1012 FORMAT(5X,2F7.3)
 1019 FORMAT(24X,10F10.4)
 1029 FORMAT(7X,'CONSTRAINED',7X,'UNCONSTRAINED'/1X,'OFF OPPOSITE',
     C3X,'OFF SAME'/4X,I3,11X,I3,11X,I3)
 1036 FORMAT(6I1)
 9000 FORMAT(1X,I2,1X,I2)
 9992 FORMAT(1X,F10.5)
 1037 FORMAT(6I2)
 1111 format(' ELAPSED TIME OF JOB  ',4(i2.2,'.'))
 2001 FORMAT(3I5,9F7.3)
C
C
      call gettim(itim1,itim2,itim3,itim4)
      msec=itim1*60*60+itim2*60+itim3
C
C  CPUTIM RETURNS THE TIME IN SECONDS SINCE THE BEGINNING OF THE PROGRAM
C
      call CPUTIM(KTIM1)
      JIO=21
      KIO=23
      NITR=0
      JAN=0
C
C  READ IN PARAMETERS:  JANLST = # OF UNIVERSAL ITERATIONS
C                       NRCALL = # OF CONTESTS READ IN FROM DISK
C                       KLSEN  = INTEGER POSITION OF FURTHEST LEFT LEGISLATOR
C                                ALLOWS SETTING LIBERAL TO NEG VALUES
C                       MAXIT  = MAXIMUM NUMBER OF ITERATIONS IN BHHH 
C                                ALGORITHM (ESSENTIALLY NUMBER OF INVERSIONS
C                                OF INFORMATION MATRIX)
C                       MAXSQZ = NUMBER OF FUNCTION EVALUATIONS IN BHHH
C                                ALGORITHM.  BECAUSE OF STEPPING, THERE ARE
C                                MORE FUNCTION EVALUATIONS THAN THERE ARE
C                                ITERATIONS
C                       NS     = NUMBER OF DIMENSIONS TO BE ESTIMATED
C                       NQ     = NUMBER OF CHOICES (2 FOR ROLL CALL MODEL)
C                       JNE    = NUMBER OF CHARACTERS IN READ OF ROLL CALL
C                                MATRIX FORMAT STATEMENT (SEE BELOW)
C
C  ******** EXAMPLE OF CONTROL CARD INPUT ********
C
C          NOMINAL UNIDIMENSIONAL UNFOLDING OF 1981-82 SENATE
C              3  966    3
C             30   15    1    2   19
C           15.0000  0.5000  0.40
C            0.0500   10
C          (4X,2A1,3X,A1,2X,5A1,2X,11A1,966I1)
C  ***********************************************
C
      READ(4,'(A)')TITLE
      READ(4,1001)JANLST,NRCALL,KLSEN,KNOL
      READ(4,1001)MAXIT,MAXSQZ,NS,NQ,jne
      WRITE(JIO,'(A)')TITLE
      WRITE(JIO,1001)JANLST,NRCALL,KLSEN,KNOL
      WRITE(JIO,1001)MAXIT,MAXSQZ,NS,NQ,jne
      WRITE(*,'(A)')TITLE
      WRITE(*,1001)JANLST,NRCALL,KLSEN
      WRITE(*,1001)MAXIT,MAXSQZ,NS,NQ,jne
      WRITE(KIO,'(A)')TITLE
C
C  READ IN STARTING VALUE FOR BETA AND FOR ESTIMATING
C  THE UTILITY FUNCTION. THE LEGISLATOR AND ROLL CALL STARTING VALUES ARE
C  GENERATED INTERNALLY IN THE PROGRAM.
C
      READ(4,1003)(BBB(J),J=1,2),TOLB
      WRITE(JIO,1003)(BBB(J),J=1,2),TOLB
      WRITE(*,1003)(BBB(J),J=1,2),TOLB
C
C  XVMIN IS THE CUTOFF VALUE FOR MINORITY PERCENTAGE OF THE ROLL CALL (USUALLY
C    .05 OR .025) AND KVMIN IS THE MINIMUM NUMBER OF VOTES FOR A LEGISLATOR
C    TO BE INCLUDED IN THE ANALYSIS
C
      READ(4,1006)XVMIN,KVMIN
      WRITE(JIO,1006)XVMIN,KVMIN
      WRITE(*,1006)XVMIN,KVMIN
C
C  READ IN FORMAT OF ROLL CALL MATRIX (SEE EXAMPLE ABOVE)
C
      READ(4,1004)FMT
      WRITE(*,1005)FMT
      BBB(3)=0.5
      KPTSUM=0
      NOB=0
      NBB=0
C
C  READ IN ROLL CALL DATA FROM UNIT 3 USING FORMAT STATEMENT READ IN
C     ABOVE.  NOTE THAT JNE = # A1'S IN FORMAT STATEMENT
C
C     KNAME DESCRIBES NAME, PARTY, STATE ETC. OF THE SENATOR.
C     KDATA IS ROLL CALL DATA.
C
  10  READ(3,FMT,END=20)(KNAME(NOB+1,J),J=1,jne),
     C                      (LDATA(NBB+1,J),J=1,NRCALL)
      NBB=NBB+1
      ISENS(NBB)=1
      NOB=NOB+1
      GO TO 10
  20  NOBS=NOB
      CALL START
      do 22 i = 1,nrcall
      ZSAVE(I,2,1)=ZMID(I)
      QSAVE(I,1,1)=ZMID(I)
      CSAVE(I,2)=DYN(I)
      RSAVE(I,1)=DYN(I)
  22  continue
      DO 18 KKK=1,NOBS
      XSAVE(KKK,2,1)=XDATA(KKK)
      SSAVE(KKK,1,1)=XDATA(KKK)
  18  CONTINUE
      call CPUTIM(KTIM1)
      WRITE(JIO,1008)KTIM1,KTIM1
      WRITE(*,1008)KTIM1,KTIM1
      LTIME=KTIM1      
      NTIME(1)=LTIME
C
C  **UNIVERSAL LOOP**
C     3-STEP ESTIMATION; BETA, ROLL CALLS, LEGISLATORS
C     JANLST IS # OF UNIVERSAL ITERATIONS.
C
C
      KKKK=0
      DO 9999 IIII=1,JANLST
      JAN=JAN+1
      DO 6666 NSTEP=0,2
C
C     UTILITY FUNCTION PHASE--BETA ESTIMATED 
C
       IF(NSTEP.EQ.0)THEN
            NOPAR=2
            NITR=NITR+1
            call MAXLIK
       ENDIF
C
C     CONTEST PHASE--MIDPOINT AND DISTANCE BETWEEN TWO CANDIDATES ESTIMATED
C
      IF(NSTEP.EQ.1)THEN
           NOPAR=2
           NITR=NITR+1
           call MAXLIK
           call OUTWRT(NRCALL,NSTEP,JIO,KIO,JAN,9999,KPJP,YBIGL)
           call CORR(ALP,BTA,RR,1)
      ENDIF
C
C     PAC PHASE--COORDINATE, UTILITY THRESHOLD, AND INCUMBENCY ESTIMATED
C
      IF(NSTEP.EQ.2)THEN
           NOPAR=3
           NITR=NITR+1
           call MAXLIK
           call OUTWRT(NOBS,NSTEP,JIO,KIO,JAN,9999,KPJP,YYBIGL)
           call CORR(ALP,BTA,RR,2)
      ENDIF
 6666 CONTINUE
C
C  CALCULATE NUMBER OF CONSTRAINED ROLL CALLS AND CORRELATION BETWEEN 
C  CURRENT PARAMETER ESTIMATES AND THOSE FROM PREVIOUS ITERATIONS
C
      KRSCLL=NRCALL-KBAD1-KBAD2
      call CORR(ALP,BTA,RR,0)
      WRITE(KIO,2001)KBAD2,KBAD1,KRSCLL,SPRM,SPRSD,ALP(1),BTA(1),RR(1),
     CSSS(1),SSS(3),sss(4),SSS(2)
      WRITE(*,2001)KBAD2,KBAD1,KRSCLL,SPRM,SPRSD,ALP(1),BTA(1),RR(1),
     CSSS(1),SSS(3),sss(4),SSS(2)
 9999 CONTINUE
      
C
C  END OF UNIVERSAL LOOP
C    SETUP PARAMETERS FOR FINAL ESTIMATE OF BETA AND ROLL CALLS USING
C    FINAL ADJUSTED LEGISLATOR COORDINATES
C
C  FINAL ESTIMATION OF BETA
C
      JAN=JAN+1
      JANLST=1
      NSTEP=0
      NOPAR=2
      NITR=NITR+1
      call MAXLIK
C
C  FINAL ESTIMATION OF CONTEST PARAMETERS
C
      NSTEP=1
      NOPAR=2
      NITR=NITR+1
      call MAXLIK
      call OUTWRT(NRCALL,NSTEP,JIO,KIO,JAN,9999,KPJP,YBIGL)
      call CORR(ALP,BTA,RR,1)
      call CORR(ALP,BTA,RR,0)
C
C  SETUP PARAMETERS TO CALCULATE CLASSIFICATIONS FOR PACS
C
      NSTEP=2
      JANLST=JAN
      DO 3 II=1,NOBS
      LMO(II)=II
      JJ=II
      B(1)=XDATA(II)
      B(2)=UCUT(II)
      B(3)=UINC(II)
C
C     SUBROUTINE CROSS COMPUTES 2x2 PREDICTIONS FOR ROLL CALLS AND LEGS.
C
      call cross(JJ,LL)
      JJ=1
      DO 4 I=1,3
      DO 4 J=1,3
      KPJP(II,JJ)=LL(I,J)
  4   JJ=JJ+1
  3   CONTINUE
C
C  WRITE OUT FINAL ESTIMATE OF LEGISLATORS WITH CROSS CLASSIFICATIONS
C
      call OUTWRT(NOBS,NSTEP,KIO,KIO,999,999,KPJP,YYBIGL)
C
C  SETUP PARAMETERS TO CALCULATE CLASSIFICATIONS FOR CONTESTS
C
      NSTEP=1
      DO 5 II=1,NRCALL
      JJ=II
      B(2)=DYN(II)
      B(1)=ZMID(II)
      call cross(JJ,LL)
      JJ=1
      DO 6 I=1,3
      DO 6 J=1,3
      KPJP(II,JJ)=LL(I,J)
  6   JJ=JJ+1
  5   CONTINUE
C
C  WRITE OUT FINAL ESTIMATES OF CONTESTS WITH CROSS CLASSIFICATIONS
C
      call OUTWRT(NRCALL,NSTEP,KIO,KIO,999,999,KPJP,YBIGL)
C
C  WRITE OUT FINAL NUMBERS OF CONSTRAINED CONTESTS
C
C      KRSCLL=NRCALL-KBAD1-KBAD2
C      WRITE(JIO,1029)KBAD2,KBAD1,KRSCLL
C      WRITE(*,1029)KBAD2,KBAD1,KRSCLL
C
C  SORT PACS AND WRITE OUT THE 4 MOST EXTREME PACS
C
      call RSORT(XDATA,NOBS,LMO)
      AA=XDATA(1)
      IA=1
      AB=XDATA(2)
      IB=2
      AC=XDATA(NOBS-1)
      IC=NOBS-1
      AD=XDATA(NOBS)
      ID=NOBS
      WRITE(JIO,1039)IA,(KNAME(LMO(1),J),J=1,25),AA,IB,(KNAME(LMO(2),J),
     CJ=1,25),AB,IC,(KNAME(LMO(IC),J),J=1,25),AC,ID,(KNAME(LMO(ID),J),
     CJ=1,25),AD
      WRITE(*,1039)IA,(KNAME(LMO(1),J),J=1,25),AA,IB,(KNAME(LMO(2),J),
     CJ=1,25),AB,IC,(KNAME(LMO(IC),J),J=1,25),AC,ID,(KNAME(LMO(ID),J),
     CJ=1,25),AD
C      CALL OUTPROD
 1039 FORMAT(' SAG IN RECOVERED PAC COORDINATES'/3(1X,I4,1X,25A1,
     C1X,F7.3/),1X,I4,1X,25A1,1X,F7.3)
      call gettim(jtim1,jtim2,jtim3,jtim4)
      write(JIO,1000)itim1,itim2,itim3,itim4
      write(JIO,1111)jtim1,jtim2,jtim3,jtim4
      write(*,1000)itim1,itim2,itim3,itim4
      write(*,1111)jtim1,jtim2,jtim3,jtim4
      STOP
      END
C
C  **************************************************************************
C    SUBROUTINE MAXLIK---CALLED BY MAIN.  INITIALIZES VARIABLES FOR 
C          ESTIMATION OF PARAMETERS
C  **************************************************************************
C
      SUBROUTINE MAXLIK
      DIMENSION LL(4,4),MM(4,4)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /ROLL/KPJP(1601,10)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
      COMMON /BOOK/ BBB(50)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),XINC(1601),RC(500)
      EXTERNAL LOGLIK,FUNNEL
  23  FORMAT(' TOTAL VOTES PREDICTED FOR',I5,1X,'CANDIDATES'/13X,
     C'ACTUAL'/7X,' CHALL    ABST    INCUM')
  24  FORMAT(' TOTAL VOTES PREDICTED FOR',I7,1X,'PACS'/13X,
     C'ACTUAL'/7X,' CHALL     ABST    INCUM')
  25  FORMAT(' CHALL',I7,2X,I7,2X,I7/' ABST',I8,2X,I7,2X,I7/' INCUM',
     CI7,2X,I7,2X,I7)
1666  FORMAT(2F7.3)
C
C  ***** UTILITY FUNCTION PHASE *****
C
      IF(NSTEP.EQ.0)THEN
           B(1)=BBB(1)
           B(2)=BBB(3)
           NFEVAL=0
           call BHHH(FUNNEL,F)
           call RPRINT(F,B,V)
      ENDIF
C
C  ***** CONTEST PHASE *****
C
      IF(NSTEP.EQ.1)THEN
           DO 22 I=1,3
           DO 22 J=1,3
  22       MM(I,J)=0
           NEQ=0
  1        CONTINUE
           NEQ=NEQ+1
           NFEVAL=0
           B(2)=DYN(NEQ)
           B(1)=ZMID(NEQ)
           call BHHH(FUNNEL,F)
           call RPRINT(F,B,V)
           call cross(NEQ,LL)
           JJ=1
           DO 21 I=1,3
           DO 21 J=1,3
           KPJP(NEQ,JJ)=LL(I,J)
           JJ=JJ+1
  21       MM(I,J)=MM(I,J)+LL(I,J)
           IF(NEQ.LT.NRCALL)GO TO 1
           WRITE(KIO,23)NRCALL
           WRITE(KIO,25)((MM(I,J),J=1,3),I=1,3)
      ENDIF
C
C  ***** PAC PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           DO 26 I=1,3
           DO 26 J=1,3
  26       MM(I,J)=0
           NEQ=0
  10       CONTINUE
           NEQ=NEQ+1
           NFEVAL=0
           B(1)=XDATA(NEQ)
           B(2)=UCUT(NEQ)
           B(3)=UINC(NEQ)
           call BHHH(FUNNEL,F)
           call RPRINT(F,B,V)
           call cross(NEQ,LL)
           JJ=1
           DO 27 I=1,3
           DO 27 J=1,3
           KPJP(NEQ,JJ)=LL(I,J)
           JJ=JJ+1
  27       MM(I,J)=MM(I,J)+LL(I,J)
           IF(NEQ.LT.NOBS)GO TO 10
           WRITE(KIO,24)NOBS
           WRITE(KIO,25)((MM(I,J),J=1,3),I=1,3)
      CALL NORMZ
      ENDIF
      RETURN
      END
C
C  **************************************************************************
C    SUBROUTINE BHHH--CALLED BY MAXLIK.  IMPLEMENTS THE BHHH ALGORITHM FOR  
C       MAXIMIZING LOG-LIKELIHOOD
C  **************************************************************************
C
      SUBROUTINE BHHH(FUNCT,XLNL)
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /LIKE/ SVLNL,FNLNL,ICNVG,ITT,NVAL
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /TOL/ TOLB
      DIMENSION AV(25,25),KCONST(20),
     czmat(25,25),wvec(25),fv1(25),fv2(25)
      EXTERNAL FUNCT
  101 format(' PERFORMANCE INDEX EIGENVECTOR/VALUE ROUTINE=',I4)
C
  102 FORMAT(1X,2I4,1X,3E20.12)
      ICNVG=0
      DO 50 I=1,NOPAR
      KCONST(I)=1
      DELTAB(I)=0.0
   50 TESTB(I)=B(I)

C
C          BERNDT, HALL, HALL, AND HAUSMAN METHOD
C
      DO 1000 IT=1,MAXIT
      IFDRV=1
C      
      XLNL=FUNCT(0.0)
C
      IF(IT.EQ.1)SVLNL=-XLNL
      IFDRV=0
      IF(NOPAR.EQ.1)GO TO 20
      IF(NSTEP.EQ.0) KCONST(1)=0
      IF(NSTEP.EQ.0) KCONST(2)=1
      IF(NSTEP.EQ.0) KCONST(3)=0
      IF(NSTEP.EQ.1) KCONST(1)=1
      IF(NSTEP.EQ.1.AND.KNOL.EQ.1)KCONST(1)=0
      IF(NSTEP.EQ.1) KCONST(2)=1
      IF(NSTEP.EQ.1) KCONST(3)=0
      IF(NSTEP.EQ.2) KCONST(1)=1
      IF(NSTEP.EQ.2) KCONST(2)=1
      IF(NSTEP.EQ.2) KCONST(3)=0
      MSUM=KCONST(1)+KCONST(2)+KCONST(3)
      IF(MSUM.EQ.0)GO TO 1200
      IF(MSUM.EQ.1)GO TO 21
      IF(MSUM.EQ.2)GO TO 23
C     
C         V11 V12 V13
C     V=  V21 V22 V23
C         V31 V32 V33
C
C     AV = INVERSE-V
C
C  CALCULATE MATRIX INVERSE
C
      call rs(25,3,v,wvec,1,ZMAT,fv1,fv2,ier)
      DO 60 I=1,3
      DO 60 KX=1,3
      SUM=0.0
      DO 61 J=1,3
      IF(WVEC(J).EQ.0)XWVEC=0.0
      IF(WVEC(J).NE.0)XWVEC=1.0/WVEC(J)
  61  SUM=SUM+ZMAT(KX,J)*(XWVEC)*ZMAT(I,J)
  60  av(I,KX)=SUM
      DET=V(1,1)*V(2,2)*V(3,3)-V(1,1)*V(2,3)*V(3,2)+V(1,2)*V(2,3)*V(3,1)
      DET=DET-V(1,2)*V(2,1)*V(3,3)+V(1,3)*V(2,1)*V(3,2)-V(1,3)*V(2,2)*
     CV(3,1)
      IF(ABS(DET).LE..00000001)ICNVG=9
      IF(ABS(DET).LE..00000001)RETURN
      DELTAB(1)=(AV(1,1)*GRAD(1)+AV(2,1)*GRAD(2)+AV(3,1)*GRAD(3))
      DELTAB(2)=(AV(1,2)*GRAD(1)+AV(2,2)*GRAD(2)+AV(3,2)*GRAD(3))
      DELTAB(3)=(AV(1,3)*GRAD(1)+AV(2,3)*GRAD(2)+AV(3,3)*GRAD(3))
      DO 77 I=1,3
      DO 77 J=1,3
      V(I,J)=AV(J,I)
77    CONTINUE
      GO TO 22
C
C     WHEN ONE OF THE THREE KCONST'S IS 1.
C
21    IF(KCONST(1).EQ.1.AND.KCONST(2).EQ.0.AND.KCONST(3).EQ.0)GO TO 30
      IF(KCONST(1).EQ.0.AND.KCONST(2).EQ.1.AND.KCONST(3).EQ.0)GO TO 31
      IF(KCONST(1).EQ.0.AND.KCONST(2).EQ.0.AND.KCONST(3).EQ.1)GO TO 32
30    IC=1
      JC=2
      KC=3
      GO TO 35
31    IC=2
      JC=1
      KC=3
      GO TO 35
32    IC=3
      JC=1
      KC=2
35    IF(ABS(V(IC,IC)).LE..00000001)ICNVG=9
      IF(ABS(V(IC,IC)).LE..00000001)RETURN
      DELTAB(IC)=GRAD(IC)/V(IC,IC)
      V(IC,IC)=1.0/V(IC,IC)
      DELTAB(JC)=0.0
      DELTAB(KC)=0.0
      GO TO 22
C
C     WHEN TWO OF THE THREE KCONST'S ARE 1'S.
C
23    IF(KCONST(1).EQ.1.AND.KCONST(2).EQ.1.AND.KCONST(3).EQ.0)GO TO 40
      IF(KCONST(1).EQ.1.AND.KCONST(2).EQ.0.AND.KCONST(3).EQ.1)GO TO 41
      IF(KCONST(1).EQ.0.AND.KCONST(2).EQ.1.AND.KCONST(3).EQ.1)GO TO 42
40    IC=1
      JC=2
      KC=3
      GO TO 45
41    IC=1
      JC=3
      KC=2
      GO TO 45
42    IC=2
      JC=3
      KC=1
45    DET=V(IC,IC)*V(JC,JC)-V(IC,JC)*V(JC,IC)
      IF(ABS(DET).LE..00000001)ICNVG=9
      IF(ABS(DET).LE..00000001)RETURN
      DELTAB(IC)=(V(JC,JC)*GRAD(IC)-V(IC,JC)*GRAD(JC))/DET
      DELTAB(JC)=(V(IC,IC)*GRAD(JC)-V(JC,IC)*GRAD(IC))/DET
      DELTAB(KC)=0.0
      S11=V(IC,IC)
      V(IC,IC)=V(JC,JC)/DET
      V(JC,JC)=S11/DET
      V(IC,JC)=-V(IC,JC)/DET
      V(JC,IC)=-V(JC,IC)/DET
      GO TO 22
  20  IF(ABS(V(1,1)).LE..00000001)ICNVG=9
      IF(ABS(V(1,1)).LE..00000001)RETURN
      DELTAB(1)=GRAD(1)/V(1,1)
      V(1,1)=1.0/V(1,1)
      DELTAB(2)=0.0
      DELTAB(3)=0.0
      GO TO 22
  22  CONTINUE
      COSDG=DELTAB(1)*GRAD(1)+DELTAB(2)*GRAD(2)+DELTAB(3)*
     CGRAD(3)
C
C  AFTER 3 ITERATIONS STEPSIZE DEFAULTS TO PREVIOUS STEPSIZE
C
      IF(IT.LE.3)ALAMB=1.0
      IFBETR=1
      call CEA(XLNL,XLNLNW,ALAMB,IFBETR,MAXSQZ,ISQZ,FUNCT)
C
C          PRINT ITERATION RESULTS
C
      IF(NSTEP.EQ.0)WRITE(KIO,1918)IT,(TESTB(J),J=1,NOPAR),
     CALAMB,COSDG,XLNL,XLNLNW
      IF(NSTEP.EQ.0)WRITE(*,1918)IT,(TESTB(J),J=1,NOPAR),
     CALAMB,COSDG,XLNL,XLNLNW
 1918 FORMAT(I5,12F15.4)
C
C          CHECK FOR CONVERGENCE OF LIKELIHOOD FUNCTION.
C
C
      IF(COSDG.LT.(TOLB**2))GO TO 1200
      IF(IFBETR.EQ.0)GO TO 1500
      DO 710 I=1,NOPAR
  710 B(I)=TESTB(I)
 1000 CONTINUE
      GO TO 1100
C
C          FAILURE TO CONVERGE
C
 1100 ICNVG=1
      ITT=IT
      GO TO 1206
C
C          CONVERGENCE
C
 1200 ICNVG=0
      ITT=IT
 1206 CONTINUE
 1207 DO 705 I=1,NOPAR
  705 B(I)=TESTB(I)
      XLNL=-FUNCT (0.0)
      FNLNL=XLNL
      ITT=IT
      NVAL=NFEVAL
      RETURN
C
C          FAILURE TO IMPROVE LNL
C
 1500 ICNVG=2
      DO 650 I=1,NOPAR
  650 TESTB(I)=B(I)
      GO TO 1207
      END
C
C  ***************************************************************************
C    SUBROUTINE CEA---CALLED BY BHHH.  CALCULATES STEPSIZE FOR GRADIENT
C  ***************************************************************************
C
      SUBROUTINE CEA(F,F0,R,IFOK,MAXSQZ,NSQZ,FUNCT)
C
C
      EXTERNAL FUNCT
C
C
C        IF STEPPING FORWARD REDUCES -LNL, DOUBLE THE STEP.
C        IF NOT, HALVE THE STEP.  ONCE STEPPING TAKES PLACE 
C        IN ONE DIRECTION, IT CONTINUES IN THAT DIRECTION
C        UNTIL NO FURTHER IMPROVEMENT OCCURS.
C
C
C
C  THIS SECTION INTIALIZES LOG-LIKELIHOOD--IF IT IS BETTER THAN OLD
C    VALUE THE STEP SIZE IS DOUBLED IN CODE SEGMENT BELOW.  IF IT IS
C    WORSE, THE STEP SIZE IN HALVED IN CODE SEGMENT IMMEDIATELY BELOW
C
      IFB = IFOK
      R0 = R
      NSQZ = 0
      IFOK=1
    2 F0 = FUNCT(R0)
      IF(F0.LT.F.AND.IFB.EQ.1)GO TO 10
  99  R0=R0/2.
      NSQZ = NSQZ +1
      IF(NSQZ .LE. MAXSQZ) GO TO 2
      IFOK=0
      R = R0
      RETURN
   10 IF(NSQZ .GT. 0) GO TO 13
C
C  IF IMPROVEMENT, STEP SIZE IS DOUBLED
C
   14 R1 = R0 * 2.
      F1 = FUNCT(R1)
      NSQZ = NSQZ +1
      IF(F1 .GT. F0) GO TO 11
      IF(F1.LT.0.0)GO TO 11
      R0 = R1
      F0 = F1
      IF(NSQZ .LT. MAXSQZ) GO TO 14
      R = R1
      F0 = F1
      RETURN
C
   11 F0 = FUNCT(R0)
   13 R = R0
      RETURN
      END
C
C  **************************************************************************
C    FUNCTION FUNNEL---CALLED BY CEA AND BY BHHH WHEN INITIALIZING.  IT
C      CALCULATES THE UPDATED PARAMETER VECTOR AS NOTED BELOW AND CALLS
C      SUBROUTINE LOGLIK WHICH CALCULATES THE CORRESPONDING LOG-LIKELIHOOD
C  **************************************************************************
C
      REAL FUNCTION FUNNEL(XLBA)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      EXTERNAL ITHOBS
C
C          UPDATE TRIAL PARAMETER VECTOR.
C
C        TESTB = B + XLBA * DELTAB
C
C           WHERE DELTAB = QMAT(INVERSE) * G
C           AND XLBA = STEPSIZE
C
      DO 10 I=1,NOPAR
  10  TESTB(I)=B(I)+XLBA*DELTAB(I)
      call LOGLIK(TESTB,F,GRAD,V)
      FUNNEL=F
   96 RETURN
      END
C
C  ***************************************************************************
C    SUBROUTINE LOGLIK---CALLED BY FUNNEL.  CALCULATES LOG-LIKELIHOOD.  FOR
C       EACH APPROPRIATE OBSERVATION, IT CALLS SUBROUTINE ITHOBS (ITH 
C       OBSERVATION) WHICH RETURNS THE LOG-LIKELIHOOD.  THE SUM IS CUMULATED
C       BELOW.
C  ***************************************************************************
C
      SUBROUTINE LOGLIK(B,FUNCT,GRAD,V)
C
C
      DIMENSION B(50),GRAD(50),V(25,25),G(50),XBIGL(1601)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),XINC(1601),RC(500)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /LUCK/ YBIGL(1601),YYBIGL(1601)
C
C
      IF(NSTEP.EQ.0)NNBS=NOBS
      IF(NSTEP.EQ.1)NNBS=NOBS
      IF(NSTEP.EQ.2)NNBS=NRCALL
      NFEVAL=NFEVAL+1
      FUNCT=0.0
      IF(NSTEP.EQ.2)YYBIGL(NEQ)=0.0
      IF(NSTEP.NE.1)GO TO 3
      DO 2 J=NEQ,NRCALL
  2   YBIGL(J)=0.0
  3   CONTINUE
      IF(IFDRV.EQ.0)GO TO 202
      DO 201 J=1,NOPAR
      GRAD(J)=0.0
  201 CONTINUE
      DO 101 J=1,NOPAR
      DO 101 I=1,NOPAR
  101 V(I,J)=0.0
  202 CONTINUE
C
C          LOOP OVER NUMBER OF OBSERVATIONS NNBS AND SUM LOG LIKELIHOOD
C     AND ITS GRADIENT WITH RESPECT TO B.
C
      I=0
  200 I=I+1
      IF(I.GT.NNBS)GO TO 211
      call ITHOBS(I,B,XLNL,G,XBIGL)
      IF(NSTEP.EQ.2)YYBIGL(NEQ)=YYBIGL(NEQ)+XLNL
      IF(NSTEP.EQ.1)YBIGL(NEQ)=YBIGL(NEQ)+XLNL
      XLNL=-XLNL
      FUNCT=FUNCT+XLNL
      IF(IFDRV.EQ.0)GO TO 200
      DO 205 J=1,NOPAR
  205 GRAD(J)=GRAD(J)+G(J)
C
C          EVALUATE QMAT, THE APPROXIMATION TO THE HESSIAN, AS THE
C     OUTER PRODUCT OF THE GRADIENT VECTOR ONLY FOR THE BHHH METHOD.
C
      DO 206 L=1,NOPAR
      DO 206 J=1,NOPAR
  206 V(J,L)=V(J,L)+G(J)*G(L)
      GO TO 200
  211 RETURN
      END
C
C  ***************************************************************************
C    SUBROUTINE ITHOBS--CALLED BY LOGLIK.  CALCULATES THE LOG-LIKELIHOOD FOR
C      EACH OBSERVATION (I.E. THE LOG-LIKELIHOOD FOR ONE COLUMN/ROW IN THE
C      INPUT CHOICE MATRIX.
C  ***************************************************************************
C
      SUBROUTINE ITHOBS(JT,B,XL,G,XBIGL)
      DIMENSION DSQ(4,1601),XB(4,1601),KNM(5),
     CEXB(4,1601),DLDX(4),B(50),G(50),
     CPHI(1601),XBIGL(1601),ECT(4,1601),XBA(4,1601),UECT(4,1601)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),XINC(1601),RC(500)
      COMMON /BOOK/ BBB(50)
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
9991  FORMAT(4x,E15.10,4x,e15.10,3x,e15.10)
9992  FORMAT(1X,E15.10)
C
C   SUBROUTINE TO ESTIMATE MULTINOMIAL LOGIT FOR SPATIAL MODELS
C
C   NP=NUMBER OF INDIVIDUALS=NOBS
C   NS=NUMBER OF DIMENSIONS
C   NQ=NUMBER OF STIMULI
C
C
C   B (PARAMETER VECTOR IN MAXLIK) SHOULD HAVE THE FOLLOWING
C   FORMAT:
C
C    IF NSTEP EQUALS 0
C     B(1) IS THE SIGNAL TO NOISE RATIO, BETA
C
C
C    IF NSTEP EQUALS 1
C     B(1) IS THE DISTANCE/2 BETWEEN THE TWO CANDIDATES
C     B(2) IS THE MIDPOINT OF THE CANDIDATES
C
C    IF NSTEP EQUALS 2
C     B(1) IS THE ITH PAC'S COORDINATE
C     B(2) IS THE ITH PAC'S UTILITY THRESHHOLD
C     B(3) IS THE ITH PAC'S INCUMBENT ADVANTAGE
C
C
C    LDATA(JT, ... )IS THE CHOICE VARIABLE
C        (E.G. YES=1, NO=2)
C    XDATA(JT) ARE THE INDIVIDUAL COORDINATES
C
C
C
      IF(JT.NE.1)GO TO 11
      IF(NSTEP.EQ.0)THEN
         KRCALL=1
         LRCALL=NRCALL
         BETA=B(1)
         WT=BBB(2)
      ENDIF
      IF(NSTEP.NE.0)THEN
         BETA=BBB(1)
         WT=BBB(2)
      ENDIF
  11  CONTINUE
      DO 27 JJ=1,5
      KNM(JJ)=0
  27  G(JJ)=0.0
      XL=0.0
C
C  ***** PAC PHASE *****
C    COMPUTE DERIVATIVE FOR PACS
C
C
      IF(NSTEP.EQ.2)THEN
         WTSQ=WT**2
         ICH=LDATA(NEQ,JT)
         D2I=(ABS(B(1)-ZMID(JT)+DYN(JT)))**2
         D2C=(ABS(B(1)-ZMID(JT)-DYN(JT)))**2
         XBI=-(D2I*WTSQ)/2.0
         IF(B(3).GT.1)B(3)=1.0
         IF(B(3).LT.-1)B(3)=-1.0
         XBI=BETA*EXP(XBI)*EXP(B(3))
         XBC=-(D2C*WTSQ)/2.0
         XBC=BETA*EXP(XBC)
         GAM=EXP(B(2)+BBB(3)*RC(JT))
         if(gam.gt.10.00)gam=15.00
         if(gam.le.0001)gam=.0001
         ENIII=EXP(GAM+3*XBI)
         EIIC=EXP(2*XBI+XBC)
         ENNIIC=EXP(2*GAM+2*XBI+XBC)
         ENICC=EXP(GAM+XBI+2*XBC)
         ENIIC=EXP(GAM+2*XBI+XBC)
         EICC=EXP(XBI+2*XBC)
         ENNICC=EXP(2*GAM+XBI+2*XBC)
         ENCCC=EXP(GAM+3*XBC)
         FA=ENNIIC-EIIC-EICC+ENNICC
         IF(FA.LE.0.00000001)FA=.00000001
         FI=ENIII+2*EIIC+ENICC
         FC=ENIIC+2*EICC+ENCCC
         PHIXI=ENIII+EIIC+ENNIIC+ENICC+ENIIC+EICC+ENNICC+ENCCC
         AA=ALOG(PHIXI)
         XL=-AA
C
C    LIKELIHOOD
C
         IF(ICH.EQ.1)XL=XL+ALOG(FI)
         IF(ICH.EQ.-1)XL=XL+ALOG(FC)
         IF(ICH.EQ.0)XL=XL+ALOG(FA)
C
C    DERIVATIVES
C
         DLDX(1)=B(1)-ZMID(JT)+DYN(JT)
         DLDX(2)=B(1)-ZMID(JT)-DYN(JT)
         DLDX(1)=-WTSQ*DLDX(1)*XBI
         DLDX(2)=-WTSQ*DLDX(2)*XBC
         IF(ICH.EQ.0)THEN
             G(1)=((ENNIIC+2*ENNICC-2*EICC-EIIC)*DLDX(2))/FA
             G(1)=G(1)+((2*ENNIIC+ENNICC-2*EIIC-EICC)*DLDX(1))/FA
         ENDIF
         IF(ICH.EQ.1)THEN
          G(1)=((2*EIIC+2*ENICC)*DLDX(2))/FI
          G(1)=G(1)+((3*ENIII+4*EIIC+ENICC)*DLDX(1))/FI
         ENDIF
         IF(ICH.EQ.-1)THEN
          G(1)=((2*EICC+2*ENIIC)*DLDX(1))/FC
          G(1)=G(1)+((3*ENCCC+4*EICC+ENIIC)*DLDX(2))/FC
         ENDIF
         PH1=(PHIXI+2*ENIII+EIIC+ENNIIC+ENIIC-ENCCC)*DLDX(1)
         PH2=(PHIXI+ENICC+EICC+ENNICC+2*ENCCC-ENIII)*DLDX(2)
         G(1)=G(1)-(PH1+PH2)/PHIXI
         IF(ICH.EQ.0)G(2)=2*GAM*(ENNIIC+ENNICC)/FA
         IF(ICH.EQ.1)G(2)=GAM*(ENIII+ENICC)/FI
         IF(ICH.EQ.-1)G(2)=GAM*(ENCCC+ENIIC)/FC
         G(2)=G(2)-GAM*(ENIII+2*ENNIIC+ENICC+ENIIC+2*ENNICC+ENCCC)
     C           /PHIXI
         IF(ICH.EQ.0)G(3)=((2*ENNIIC+ENNICC-2*EIIC-EICC)*XBI)/FA
         IF(ICH.EQ.1)G(3)=((3*ENIII+4*EIIC+ENICC)*XBI)/FI
         IF(ICH.EQ.-1)G(3)=((2*EICC+2*ENIIC)*XBI)/FC
         G(3)=G(3)-((PHIXI+2*ENIII+EIIC+ENNIIC+ENIIC-ENCCC)*XBI)/PHIXI
C
C  IF LEGISLATOR IS CONSTRAINED DUE TO SAG PROBLEM, SET DERIVATIVE TO ZERO
C
      IF(ISENS(NEQ).EQ.0)G(1)=0.0
      G(3)=0.0
      ENDIF
C
C  ***** CONTEST PHASE *****
C  CONTEST DERIVATIVES
C
C
      IF(NSTEP.EQ.1.AND.KNOL.NE.1)THEN
         WTSQ=WT**2
         XBIGL(NEQ)=0.0
         ICH=LDATA(JT,NEQ)
         IF(B(1).LT.-1)B(1)=-1.0
         IF(B(1).GT.1)B(1)=1.0
         IF(B(2).GT.2)B(2)=2.0
         IF(B(2).LT.-2)B(2)=-2.0
         DSQ(1,NEQ)=(XDATA(JT)-B(1)+B(2))**2
         DSQ(2,NEQ)=(XDATA(JT)-B(1)-B(2))**2
         ECT(1,JT)=EXP(UCUT(JT)+BBB(3)*RC(NEQ))
         ECT(2,JT)=UINC(JT)        
         UECT(1,JT)=EXP(ECT(1,JT))
         UECT(2,JT)=EXP(UINC(JT))
C
C   COMPUTE PHI
C
         DO 6 I = 1,2
         XB(I,NEQ)=-DSQ(I,NEQ)*WTSQ/2.0
         XB(I,NEQ)=BETA*EXP(XB(I,NEQ))
         IF(I.EQ.1)XB(I,NEQ)=XB(I,NEQ)*UECT(2,JT)
         EXB(I,NEQ) = EXP(XB(I,NEQ))
  6      CONTINUE
         XBA(1,NEQ)=WTSQ*(XDATA(JT)-B(1)+B(2))*XB(1,NEQ)
         XBA(2,NEQ)=WTSQ*(XDATA(JT)-B(1)-B(2))*XB(2,NEQ)
C
C   COMPUTE LIKELIHOOD
C        
         ENIII=EXP(ECT(1,JT)+3*XB(1,NEQ))
         EIIC=EXP(2*XB(1,NEQ)+XB(2,NEQ))
         ENNIIC=EXP(2*ECT(1,JT)+2*XB(1,NEQ)+XB(2,NEQ))
         ENICC=EXP(ECT(1,JT)+XB(1,NEQ)+2*XB(2,NEQ))
         ENIIC=EXP(ECT(1,JT)+2*XB(1,NEQ)+XB(2,NEQ))
         EICC=EXP(XB(1,NEQ)+2*XB(2,NEQ))
         ENNICC=EXP(2*ECT(1,JT)+XB(1,NEQ)+2*XB(2,NEQ))
         ENCCC=EXP(ECT(1,JT)+3*XB(2,NEQ))
         FA=ENNIIC-EIIC-EICC+ENNICC
         IF(FA.LE.0.00000001)FA=.00000001
         FI=ENIII+2*EIIC+ENICC
         FC=ENIIC+2*EICC+ENCCC
         PHI(NEQ)=ENIII+EIIC+ENNIIC+ENICC+ENIIC+EICC+ENNICC+ENCCC
         CC=ALOG(PHI(NEQ))
         XL=XL-CC
         IF(ICH.EQ.1)XL=ALOG(FI)+XL
         IF(ICH.EQ.-1)XL=ALOG(FC)+XL
         IF(ICH.EQ.0)XL=ALOG(FA)+XL
C
C    COMPUTE DERIVATIVES FOR ROLL CALL OUTCOMES
C     
         PH1=(PHI(NEQ)+2*ENIII+EIIC+ENNIIC+ENIIC-ENCCC)*XBA(1,NEQ)
         PH2=(PHI(NEQ)+ENICC+EICC+ENNICC+2*ENCCC-ENIII)*XBA(2,NEQ)
         IF(ICH.EQ.0)THEN
           G(1)=(ENNIIC+2*ENNICC-2*EICC-EIIC)*XBA(2,NEQ)/FA
           G(1)=G(1)+(2*ENNIIC+ENNICC-2*EIIC-EICC)*XBA(1,NEQ)/FA
         ENDIF
         IF(ICH.EQ.1)THEN
           G(1)=(2*EIIC+2*ENICC)*XBA(2,NEQ)/FI
           G(1)=G(1)+(3*ENIII+4*EIIC+ENICC)*XBA(1,NEQ)/FI
         ENDIF
         IF(ICH.EQ.-1)THEN
           G(1)=(ENIIC+4*EICC+3*ENCCC)*XBA(2,NEQ)/FC
           G(1)=G(1)+(2*ENIIC+2*EICC)*XBA(1,NEQ)/FC
         ENDIF 
         G(1)=G(1)-(PH1+PH2)/PHI(NEQ)
         IF(ICH.EQ.0)THEN
           G(2)=(FA+ENNICC-EICC)*XBA(2,NEQ)/FA
           G(2)=G(2)-(FA+ENNIIC-EIIC)*XBA(1,NEQ)/FA
         ENDIF
         IF(ICH.EQ.1)THEN
           G(2)=(2*EIIC+2*ENICC)*XBA(2,NEQ)/FI
           G(2)=G(2)-(3*ENIII+4*EIIC+ENICC)*XBA(1,NEQ)/FI
         ENDIF
         IF(ICH.EQ.-1)THEN
           G(2)=(ENIIC+4*EICC+3*ENCCC)*XBA(2,NEQ)/FC
           G(2)=G(2)-(2*ENIIC+2*EICC)*XBA(1,NEQ)/FC
         ENDIF 
      G(2)=G(2)-(PH2-PH1)/PHI(NEQ)
      ENDIF
C
C   METHOD II
C
       IF(NSTEP.EQ.1.AND.KNOL.EQ.1)THEN
         WTSQ=WT**2
         XBIGL(NEQ)=0.0
         ICH=LDATA(JT,NEQ)
C         IF(B(1)+B(2).LT.sss(1))B(2)=SSS(1)-B(1)
C         IF(B(1)+B(2).GT.sss(2))B(2)=SSS(2)-B(1)
         DSQ(1,NEQ)=(XDATA(JT)-XINC(NEQ))**2
         DSQ(2,NEQ)=(XDATA(JT)-XINC(NEQ)-2*B(2))**2
         ECT(1,JT)=EXP(UCUT(JT)+BBB(3)*RC(NEQ))
         ECT(2,JT)=UINC(JT)        
         UECT(1,JT)=EXP(ECT(1,JT))
         UECT(2,JT)=EXP(UINC(JT))
C
C   COMPUTE PHI
C
         DO 46 I = 1,2
         XB(I,NEQ)=-DSQ(I,NEQ)*WTSQ/2.0
         XB(I,NEQ)=BETA*EXP(XB(I,NEQ))
         IF(I.EQ.1)XB(I,NEQ)=XB(I,NEQ)*UECT(2,JT)
         EXB(I,NEQ) = EXP(XB(I,NEQ))
 46      CONTINUE
         XBA(2,NEQ)=2*WTSQ*(XDATA(JT)-XINC(NEQ)-2*B(2))*XB(2,NEQ)
C
C   COMPUTE LIKELIHOOD
C        
         ENIII=EXP(ECT(1,JT)+3*XB(1,NEQ))
         EIIC=EXP(2*XB(1,NEQ)+XB(2,NEQ))
         ENNIIC=EXP(2*ECT(1,JT)+2*XB(1,NEQ)+XB(2,NEQ))
         ENICC=EXP(ECT(1,JT)+XB(1,NEQ)+2*XB(2,NEQ))
         ENIIC=EXP(ECT(1,JT)+2*XB(1,NEQ)+XB(2,NEQ))
         EICC=EXP(XB(1,NEQ)+2*XB(2,NEQ))
         ENNICC=EXP(2*ECT(1,JT)+XB(1,NEQ)+2*XB(2,NEQ))
         ENCCC=EXP(ECT(1,JT)+3*XB(2,NEQ))
         FA=ENNIIC-EIIC-EICC+ENNICC
         IF(FA.LE.0.00000001)FA=.00000001
         FI=ENIII+2*EIIC+ENICC
         FC=ENIIC+2*EICC+ENCCC
         PHI(NEQ)=ENIII+EIIC+ENNIIC+ENICC+ENIIC+EICC+ENNICC+ENCCC
         CC=ALOG(PHI(NEQ))
         XL=XL-CC
         IF(ICH.EQ.1)XL=ALOG(FI)+XL
         IF(ICH.EQ.-1)XL=ALOG(FC)+XL
         IF(ICH.EQ.0)XL=ALOG(FA)+XL
C
C    COMPUTE DERIVATIVES FOR ROLL CALL OUTCOMES
C     
         PH2=(PHI(NEQ)+ENICC+EICC+ENNICC+2*ENCCC-ENIII)*XBA(2,NEQ)
         IF(ICH.EQ.0)G(2)=(ENNIIC+2*ENNICC-2*EICC-EIIC)*XBA(2,NEQ)/FA
         IF(ICH.EQ.1)G(2)=(2*EIIC+2*ENICC)*XBA(2,NEQ)/FI
         IF(ICH.EQ.-1)G(2)=(ENIIC+4*EICC+3*ENCCC)*XBA(2,NEQ)/FC
         G(2)=G(2)-PH2/PHI(NEQ)
      G(1)=0.0
      ENDIF
C
C  ***** UTILITY FUNCTION PHASE *****
C  DERIVATIVES FOR BETA
C
      IF(NSTEP.EQ.0)THEN
         WTSQ=WT**2
         DO 26 II=KRCALL,LRCALL
C         XBIGL(II)=0.0
         ICH=LDATA(JT,II)
         DSQ(1,II)=(XDATA(JT)-ZMID(II)+DYN(II))**2
         DSQ(2,II)=(XDATA(JT)-ZMID(II)-DYN(II))**2
         ECT(1,JT)=EXP(UCUT(JT)+B(2)*RC(II))
         ECT(2,JT)=UINC(JT)        
C
C   COMPUTE PHI
C
         UECT(1,JT)=EXP(ECT(1,JT))
         UECT(2,JT)=EXP(ECT(2,JT))
         DO 66 I=1,2
         XBA(I,II)=-DSQ(I,II)*WTSQ/2.0
         XBA(I,II)=EXP(XBA(I,II))
         IF(I.EQ.1)XBA(I,II)=XBA(I,II)*UECT(2,JT)
         XB(I,II)=BETA*XBA(I,II)
         EXB(I,II)=EXP(XB(I,II))
  66     CONTINUE
C
C   COMPUTE LIKELIHOOD
C        
         ENIII=EXP(ECT(1,JT)+3*XB(1,II))
         EIIC=EXP(2*XB(1,II)+XB(2,II))
         ENNIIC=EXP(2*ECT(1,JT)+2*XB(1,II)+XB(2,II))
         ENICC=EXP(ECT(1,JT)+XB(1,II)+2*XB(2,II))
         ENIIC=EXP(ECT(1,JT)+2*XB(1,II)+XB(2,II))
         EICC=EXP(XB(1,II)+2*XB(2,II))
         ENNICC=EXP(2*ECT(1,JT)+XB(1,II)+2*XB(2,II))
         ENCCC=EXP(ECT(1,JT)+3*XB(2,II))
         FA=ENNIIC-EIIC-EICC+ENNICC
         IF(FA.LE.0.00000001)FA=.00000001
         FI=ENIII+2*EIIC+ENICC
         FC=ENIIC+2*EICC+ENCCC
         PHI(II)=ENIII+EIIC+ENNIIC+ENICC+ENIIC+EICC+ENNICC+ENCCC
C
C    LIKELIHOOD
C
         IF(ICH.EQ.1)XL=XL+ALOG(FI)
         IF(ICH.EQ.-1)XL=XL+ALOG(FC)
         IF(ICH.EQ.0)XL=XL+ALOG(FA)
         CC=ALOG(PHI(II))
         XL=XL-CC
C
C   COMPUTE FIRST PARTIALS OF BETA
C
         IF(ICH.EQ.0)THEN
           G(1)=G(1)+(FA+ENNICC-EICC)*XBA(2,II)/FA
           G(1)=G(1)+(FA+ENNIIC-EIIC)*XBA(1,II)/FA
         ENDIF
         IF(ICH.EQ.1)THEN
           G(1)=G(1)+(2*EIIC+2*ENICC)*XBA(2,II)/FI
           G(1)=G(1)+(3*ENIII+4*EIIC+ENICC)*XBA(1,II)/FI
         ENDIF
         IF(ICH.EQ.-1)THEN
           G(1)=G(1)+(ENIIC+4*EICC+3*ENCCC)*XBA(2,II)/FC
           G(1)=G(1)+(2*ENIIC+2*EICC)*XBA(1,II)/FC
         ENDIF 
         G(1)=G(1)-(3*ENIII+2*ENNIIC+2*EIIC+ENICC)*XBA(1,II)/PHI(II)
         G(1)=G(1)-(2*ENIIC+ENNICC+EICC)*XBA(1,II)/PHI(II)
         G(1)=G(1)-(ENNIIC+EIIC+2*ENICC+ENIIC)*XBA(2,II)/PHI(II)
         G(1)=G(1)-(2*ENNICC+2*EICC+3*ENCCC)*XBA(2,II)/PHI(II)
         IF(ICH.EQ.0)G(2)=G(2)+2*RC(II)*ECT(1,JT)*(ENNIIC+ENNICC)/FA
         IF(ICH.EQ.1)G(2)=G(2)+RC(II)*ECT(1,JT)*(ENIII+ENICC)/FI
         IF(ICH.EQ.-1)G(2)=G(2)+RC(II)*ECT(1,JT)*(ENCCC+ENIIC)/FC
         G(2)=G(2)-RC(II)*ECT(1,JT)*(ENIII+2*ENNIIC+ENICC+ENIIC+2*
     C      ENNICC+ENCCC)/PHI(II)
  26     CONTINUE
      G(1)=0.0
      ENDIF
      RETURN
      END
C
C  **************************************************************************
C    SUBROUTINE RPRINT---CALLED BY MAXLIK.  STORES NEW PARAMETER ESTIMATES
C       AND (OPTIONALLY) WRITES PARAMETER ESTIMATES TO DISK.  IN ROLL CALL
C       PHASE, IF CONVERGED ESTIMATES ARE "UNREALISTIC", GRID2
C       SUBROUTINE IS CALLED TO PRODUCED "APPROPRIATE" CONSTRAINED 
C       ESTIMATES
C  **************************************************************************
C
      SUBROUTINE RPRINT(F,B,V)
      DIMENSION SSSE(25),TTTE(25),B(50),V(25,25)
      character*1 kname
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),XINC(1601),RC(500)
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
      COMMON /BOOK/ BBB(50)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /SAVE/ XSAVE(513,8,2),ZSAVE(1601,8,2),CSAVE(1601,8),
     CQSAVE(1601,8,2),RSAVE(1601,8),SSAVE(513,8,2)
      COMMON /LIKE/ SVLNL,FNLNL,ICNVG,ITT,NVAL
      COMMON /TCHOIC/ KPTSUM
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /STD/ STDDVX(513),STDDVZ(1601,3)
      COMMON /VOTES/ KAV(1601),KAY(1601),KAN(1601),KNAME(513,25),JNE
C
 1001 FORMAT(I5,2F13.4,3I4,4F9.4/34X,5F9.4/25X,6F9.4)
 1002 FORMAT(1X,2I4,2I7,F14.4,4F9.4)
 1003 FORMAT(1X,I4,1X,19A1,1X,2F13.4,3I4,4F9.4)
 1010 FORMAT(1X,I4,2F13.4,3I4,4F9.4/30X,F13.4,4F9.4)
 1011 FORMAT(I5,2F13.4,3I4,4F9.4/34X,5F9.4)
 1012 FORMAT(1X,I4,2F13.4,3I4,4F9.4/34X,5F9.4/25X,6F9.4)
      DO 10 I=1,NOPAR
      SE=SQRT(ABS(V(I,I)))
      T=0.0
      IF(SE.GT.0) T=B(I)/SE
      SSSE(I)=SE
      TTTE(I)=T
   10 CONTINUE
C
C  ***** UTILITY FUNCTION PHASE *****
C
      IF(NSTEP.EQ.0)THEN
           WRITE(KIO,1011)NEQ,SVLNL,FNLNL,ITT,
     C     NVAL,ICNVG,BBB(1),B(1),SSSE(1),TTTE(1),BBB(3),B(2),
     C     SSSE(2),TTTE(2)
           WRITE(*,1011)NEQ,SVLNL,FNLNL,ITT,
     C     NVAL,ICNVG,BBB(1),B(1),SSSE(1),TTTE(1),BBB(3),B(2),
     C     SSSE(2),TTTE(2)
           call CPUTIM(KTIM1)
           IITIME=ktim1
           NTIME(NITR)=KTIM1
           JITIME=IITIME
           IF(NITR.GT.1)JITIME=NTIME(NITR)-NTIME(NITR-1)
C
C     KPTSUM=TOTAL # OF CHOICES.
C
           KPTSUM=INT(NOBS*NRCALL)
           AA=F/FLOAT(KPTSUM)
C
C     AA=EXP(AA) IS GEOMETRIC MEAN PROBABILITY OF UTILITY FUNCTION.
C
           AA=EXP(AA)
           BBB(1)=B(1)
           BBB(3)=B(2)
           WRITE(JIO,1002)NITR,NSTEP,IITIME,JITIME,F,AA,BBB(1),BBB(3)
           WRITE(*,1002)NITR,NSTEP,IITIME,JITIME,F,AA,BBB(1),BBB(3)
      ENDIF
C
C  ***** CONTEST PHASE *****
C
      IF(NSTEP.EQ.1)THEN
          IF(KNOL.EQ.1)GO TO 211
C
C     IF THE MIDPOINT < FURTHEST LEFT LEGISLATOR, CALL GRID ROUTINE TO
C        CALCULATE CONSTRAINED ESTIMATES OF MIDPOINT AND YES OUTCOME
C        FOR ROLL CALL
C
            IF(B(1).LT.SSS(1))call GRID2(NOBS,NEQ,KIO,B,3)
C
C     IF THE MIDPOINT > FURTHEST RIGHT LEGISLATOR, CALL GRID ROUTINE TO
C        CALCULATE CONSTRAINED ESTIMATES OF MIDPOINT AND YES OUTCOME
C        FOR ROLL CALL
C
            IF(B(1).GT.SSS(2))call GRID2(NOBS,NEQ,KIO,B,2)
C
C     IF THE YES OUTCOME IS < FURTHEST LEFT LEGISLATOR AND YES
C         OUTCOME IS > FURTHEST RIGHT LEGISLATOR THEN CALL GRID ROUTINE
C         TO CONSTRAIN THE YES/NO OUTCOMES BUT RETAIN MIDPOINT
C                  IF DYN(.) > 0
C
            IF((B(1)-B(2)).LT.SSS(1).AND.(B(2)+B(1)).GT.SSS(2))
     C                         call GRID2(NOBS,NEQ,KIO,B,1)
C
C                  IF DYN(.) < 0
C
            IF((B(2)+B(1)).LT.SSS(1).AND.(B(1)-B(2)).GT.SSS(2))
     C                         call GRID2(NOBS,NEQ,KIO,B,1)
C
 211       WRITE(*,1010)NEQ,SVLNL,FNLNL,ITT,NVAL,ICNVG,
     C     DYN(NEQ),B(2),SSSE(2),TTTE(2),V(2,1),ZMID(NEQ),B(1),
     C     SSSE(1),TTTE(1)
C           WRITE(KIO,1010)NEQ,SVLNL,FNLNL,ITT,NVAL,ICNVG,
C     C     DYN(NEQ),B(2),SSSE(2),TTTE(2),V(2,1),ZMID(NEQ),B(1),
C     C     SSSE(1),TTTE(1)
C
           IF(KNOL.eq.1.and.B(1)+B(2).LT.sss(1))B(2)=SSS(1)-B(1)
           IF(KNOL.EQ.1.AND.B(1)+B(2).GT.sss(2))B(2)=SSS(2)-B(1)
           DYN(NEQ)=B(2)
           IF(KNOL.NE.1)ZMID(NEQ)=B(1)
           IF(KNOL.EQ.1)ZMID(NEQ)=XINC(NEQ)+DYN(NEQ)
           STDDVZ(NEQ,1)=SSSE(1)
           STDDVZ(NEQ,2)=SSSE(2)
           STDDVZ(NEQ,3)=V(1,2)
           ZSAVE(NEQ,1,1)=ZSAVE(NEQ,2,1)
           ZSAVE(NEQ,2,1)=B(1)
           CSAVE(NEQ,1)=CSAVE(NEQ,2)
           CSAVE(NEQ,2)=B(2)
           ZSAVE(NEQ,2,2)=FNLNL
           ZSAVE(NEQ,1,2)=SVLNL
      ENDIF
C
C  ***** PAC PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           WRITE(*,1012)NEQ,SVLNL,FNLNL,ITT,NVAL,ICNVG,
     C     XDATA(NEQ),B(1),SSSE(1),TTTE(1),V(2,1),UCUT(NEQ),B(2),
     C     SSSE(2),TTTE(2),V(3,1),V(3,2),UINC(NEQ),B(3),SSSE(3),TTTE(3)
           XDATA(NEQ)=B(1)
           UCUT(NEQ)=B(2)
           UINC(NEQ)=B(3)
           XSAVE(NEQ,1,1)=XSAVE(NEQ,2,1)
           XSAVE(NEQ,2,1)=B(1)
           XSAVE(NEQ,2,2)=FNLNL
           XSAVE(NEQ,1,2)=SVLNL
           STDDVX(NEQ)=SSSE(1)
      ENDIF
      RETURN
      END
C
C  ***************************************************************************
C     SUBROUTINE CROSS---CALLED BY MAXLIK AND MAIN.  CALCULATES CROSS 
C        CLASSIFICATIONS USING PREDICTED YES AND NO PROBABILITIES VERSUS
C        ACTUAL CHOICES.
C  ***************************************************************************
C
      SUBROUTINE cross(II,LL)
      DIMENSION L(4),LL(4,4)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),XINC(1601),RC(500)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(1000),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ,KNOL,KSTRT
      COMMON /BOOK/ BBB(50)
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
 9898 FORMAT(3I7)
 9899 FORMAT(3F10.5)
      BETA=BBB(1)
      WT=BBB(2)
      WTSQ=.5*(WT**2)
      DO 5 I=1,3
      L(I)=0
      DO 5 J=1,3
  5   LL(I,J)=0
C
C  ***** UTILITY FUNCTION PHASE *****
C
      IF(NSTEP.EQ.0)THEN
           DO 100 I=1,NOBS
           ICH=LDATA(I,II)
           L(ICH+2)=L(ICH+2)+1
           SUMI=WTSQ*(XDATA(I)-ZMID(II)+DYN(II))**2
           SUMC=WTSQ*(XDATA(I)-ZMID(II)-DYN(II))**2
           SUMI=BETA*EXP(-SUMI)*EXP(UINC(I))
           SUMC=BETA*EXP(-SUMC)
           ESUMI=EXP(SUMI)
           ESUMC=EXP(SUMC)
           SUMK=EXP(EXP(UCUT(I)+BBB(3)*RC(II)))
           PIA=(SUMK*ESUMC)/(ESUMI+SUMK*ESUMC)
           PIA=PIA - ESUMC/(SUMK*ESUMI+ESUMC)
           PII=ESUMI/(ESUMI+ESUMC)
           PII=PII*(1.0-PIA)
           PIC=(1.0-PII)*(1-PIA)
           IF(PII.GE.PIC.AND.PII.GE.PIA)LL(3,ICH+2)=LL(3,ICH+2)+1
           IF(PIA.GT.PII.AND.PIA.GE.PIC)LL(2,ICH+2)=LL(2,ICH+2)+1
           IF(PIC.GT.PII.AND.PIC.GT.PIA)LL(1,ICH+2)=LL(1,ICH+2)+1
  100      CONTINUE
      ENDIF
C
C  ***** ROLL CALL PHASE *****
C
      IF(NSTEP.EQ.1)THEN
           DO 101 I=1,NOBS
           ICH=LDATA(I,II)
           L(ICH+2)=L(ICH+2)+1
           SUMI=WTSQ*(XDATA(I)-ZMID(II)+DYN(II))**2
           SUMC=WTSQ*(XDATA(I)-ZMID(II)-DYN(II))**2
           SUMI=BETA*EXP(-SUMI)*EXP(UINC(I))
           SUMC=BETA*EXP(-SUMC)
           ESUMI=EXP(SUMI)
           ESUMC=EXP(SUMC)
           SUMK=EXP(EXP(UCUT(I)+BBB(3)*RC(II)))
           PIA=(SUMK*ESUMC)/(ESUMI+SUMK*ESUMC)
           PIA=PIA - ESUMC/(SUMK*ESUMI+ESUMC)
           PII=ESUMI/(ESUMI+ESUMC)
           PII=PII*(1.0-PIA)
           PIC=(1.0-PII)*(1-PIA)
           IF(PII.GE.PIC.AND.PII.GE.PIA)LL(3,ICH+2)=LL(3,ICH+2)+1
           IF(PIA.GT.PII.AND.PIA.GE.PIC)LL(2,ICH+2)=LL(2,ICH+2)+1
           IF(PIC.GT.PII.AND.PIC.GT.PIA)LL(1,ICH+2)=LL(1,ICH+2)+1
  101      CONTINUE
      ENDIF
C
C  ***** LEGISLATOR PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           DO 200 I=1,NRCALL
           ICH=LDATA(II,I)
C
C    USE THIS STATEMENT IF, ON THE LAST ITERATION, YOU DESIRE TO CALCULATE
C      THE CLASSIFICATIONS OF UNCONSTRAINED ROLL CALLS ONLY
C           if(abs(ZMID(i)).eq.1.0.AND.JANLST.EQ.JAN)go to 200
C
           L(ICH+2)=L(ICH+2)+1
           SUMI=WTSQ*(XDATA(II)-ZMID(I)+DYN(I))**2
           SUMC=WTSQ*(XDATA(II)-ZMID(I)-DYN(I))**2
           SUMI=BETA*EXP(-SUMI)*EXP(UINC(II))
           SUMC=BETA*EXP(-SUMC)
           ESUMI=EXP(SUMI)
           ESUMC=EXP(SUMC)
           SUMK=EXP(EXP(UCUT(II)+BBB(3)*RC(I)))
           PIA=(SUMK*ESUMC)/(ESUMI+SUMK*ESUMC)
           PIA=PIA - ESUMC/(SUMK*ESUMI+ESUMC)
           PII=ESUMI/(ESUMI+ESUMC)
           PII=PII*(1.0-PIA)
           PIC=(1.0-PII)*(1-PIA)
           IF(PII.GE.PIC.AND.PII.GE.PIA)LL(3,ICH+2)=LL(3,ICH+2)+1
           IF(PIA.GT.PII.AND.PIA.GE.PIC)LL(2,ICH+2)=LL(2,ICH+2)+1
           IF(PIC.GT.PII.AND.PIC.GT.PIA)LL(1,ICH+2)=LL(1,ICH+2)+1
  200      CONTINUE
      ENDIF
      RETURN
      END
C
C  ***************************************************************************
C    SUBROUTINE OUTWRT---CALLED BY MAIN DURING THE ROLL CALL AND LEGISLATOR
C      PHASES ONLY.  CALCULATES NUMBER OF CONSTRAINED ROLL CALLS AND GMP 
C      FOR EACH ROLL CALL DURING ROLL CALL PHASE AND, AT THE LAST ITERATION,
C      WRITES ROLL CALL ESTIMATES TO DISK.  FOR LEGISLATOR PHASE, THE GMP
C      FOR EACH LEGISLATOR IS CALCULATED AND, AT THE LAST ITERATION, WRITES
C      LEGISLATOR ESTIMATES TO DISK.  FOR BOTH ROLL CALL AND LEGISLATOR 
C      PHASES, THE CPU TIME OF THE PHASE IS CALCULATED, WRITTEN OUT, AND
C      STORED.
C  ***************************************************************************
C
      SUBROUTINE OUTWRT(NRX,NSTEP,JIO,KIO,JAN,JANLST,KPJP,YBIGL)
      DIMENSION YBIGL(1601),KPJP(1601,10)
      character*1 kname
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(501,601),
     CUCUT(501),UINC(501),XINC(1601),RC(500)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /VOTES/ KAV(1601),kay(1601),kan(1601),kname(513,25),jne
      COMMON /STD/ STDDVX(513),STDDVZ(1601,3)
      COMMON /BOOK/ BBB(50)
      COMMON /BAD/KBAD1,KBAD2,SPRM,SPRSD
 1002 FORMAT(1X,I4,9i4,1x,7F7.3)
 1003 FORMAT(1X,2I4,2I7,F14.4,F9.4,I8)
 1004 FORMAT(1X,I4,1X,6F7.3)
 1005 FORMAT(1X,I4,1X,19A1,4I4,4F10.4)
 1014 FORMAT(1X,2I7,2G20.13)
 1015 format(1x,i4,5x,25A1,4x,5f7.3,9I4)
C
C  ***** ROLL CALL PHASE *****
C
      IF(NSTEP.EQ.1)THEN
           KBAD1=0
           KBAD2=0
           SPRM=0.0
           SPRSD=0.0
           SUM=0.0
           KKSUM=0
           DO 11 I=1,NRX
           KSUM=0
           SPRM=SPRM+ABS(2.0*DYN(I))
           SPRSD=SPRSD+(2.0*DYN(I))**2
           DO 12 JJ=1,9
  12       KSUM=KSUM+KPJP(I,JJ)
           KKSUM=KKSUM+KSUM
           SUM=SUM+YBIGL(I)
           IF(KSUM.EQ.0)AA=99.0
           IF(KSUM.EQ.0)GO TO 16
           AA=YBIGL(I)/FLOAT(KSUM)
           AA=EXP(AA)
  16       CONTINUE
           IF(JAN.LT.JANLST)GO TO 11
C           WRITE(31,1002)I,(KPJP(I,J),J=1,9),AA,DYN(I),
C     C     ZMID(I),(STDDVZ(I,J),J=1,2)
           XINCI=ZMID(I)-DYN(I)
           CHALL=ZMID(I)+DYN(I)
           WRITE(33,1004)I,AA,XINCI,CHALL,(STDDVZ(I,J),J=1,2)
  11       CONTINUE
C
C   SPRM IS THE AVERAGE DISTANCE BETWEEN THE YES AND NO OUTCOMES AND
C      SPRSD IS THE STANDARD DEVIATION OF THE DISTANCE
C
           SPRSD=SQRT(FLOAT(NRX)*SPRSD-SPRM*SPRM)/
     C     FLOAT(NRX)
           SPRM=SPRM/FLOAT(NRX)
      ENDIF
C
C ***** PAC PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           SUM=0.0
           KKSUM=0
           DO 13 I=1,NRX
           KSUM=0
           DO 14 JJ=1,9
  14       KSUM=KSUM+KPJP(I,JJ)
           KKSUM=KKSUM+KSUM
           SUM=SUM+YBIGL(I)
           IF(KSUM.EQ.0)AA=99.0
           IF(KSUM.EQ.0)GO TO 166
           AA=YBIGL(I)/FLOAT(KSUM)
           AA=EXP(AA)
 166       CONTINUE
           IF(JAN.LT.JANLST)GO TO 13
           WRITE(31,1015)I,(KNAME(I,J),J=1,25),AA,xdata(i),ucut(i),
     C     uINc(i),STDDVX(I),(KPJP(I,J),J=1,9)
  13       CONTINUE
      ENDIF
C
C  CALCULATE CPU TIME OF CURRENT PHASE
C
      call CPUTIM(KTIM1)
      IITIME=KTIM1
      IF(NITR.EQ.1)JITIME=IITIME-NTIME(1)
      NTIME(NITR)=IITIME
      IF(NITR.GT.1)JITIME=NTIME(NITR)-NTIME(NITR-1)
      AA=SUM/FLOAT(KKSUM)
      AA=EXP(AA)
      WRITE(KIO,1014)NRX,KKSUM,SUM,AA
      WRITE(JIO,1003)NITR,NSTEP,IITIME,JITIME,SUM,AA
      IF(JIO.NE.KIO)WRITE(*,1003)NITR,NSTEP,IITIME,JITIME,SUM,AA
      RETURN
      END
