C************************************************************************
C************************************************************************
C*                                                                     **
C*           1-NOMINATE                                                **
C*           ONE-DIMENSIONAL NOMINAL THREE-STEP ESTIMATION             **
C*                           ******  *          *                      **
C*           DEVELOPED BY KEITH T. POOLE AND HOWARD ROSENTHAL          **
C*           CARNEGIE-MELLON UNIVERSITY, PITTSBURGH, PA.               **
C*                                                                     **
C*           INITIAL PROGRAM DEVELOPED 1982-1984                       **
C*           THIS VERSION CURRENT AS OF JULY 1991                      **
C************************************************************************
C************************************************************************
C
C     THE PARAMETERS BEING ESTIMATED ARE: A SIGNAL TO NOISE RATIO CONSTANT   
C     IN UTILITY FUNCTION PHASE. LEGISLATOR COORDINATES (IN LEGISLATOR PHASE).
C     MIDPOINT AND DISTANCE BETWEEN THE TWO OUTCOMES IN THE ROLL CALL PHASE. 
C     ALL PARAMETERS ARE HELD FIXED EXCEPT DURING THEIR RESPECTIVE PHASES.
C     DURING UTILITY PHASE, LEGISLATOR AND ROLL CALL PARAMETERS ARE HELD FIXED
C     AND ARE STORED IN XDATA(.) AND ZMID(.)/DYN(.) RESPECTIVELY.  DURING
C     ROLL CALL PHASE, THE UTILITY FUNCTION CONSTANT, STORED IN BBB(1), IS
C     HELD FIXED AND THE LEGISLATORS ARE HELD FIXED.  DURING LEGISLATOR PHASE,
C     UTILITY AND ROLL CALL PARAMETERS ARE HELD FIXED.
C
C     THE FOLLOWING TABLE SHOWS WHAT IS STORED WHERE AND WHAT IS HELD CONSTANT
C     DURING EACH PHASE.
C
C	NOBS=# OF LEGISLATORS, NRCALL=# OF ROLL CALLS.
C
C                     UTILITY PHASE     LEGISLATOR PHASE   ROLL CALL PHASE
C
C     BETA (SIG/NOISE)   B(1)               BBB(1)             BBB(1)
C     
C     LEGISLATOR         XDATA(1) TO        B(1)               XDATA(1) TO
C       COORDS           XDATA(NOBS)                           XDATA(NOBS)
C
C     DISTANCE BET.      DYN(1) TO          DYN(1) TO          B(1)
C       OUTCOMES         DYN(NRCALL)        DYN(NRCALL)
C
C     MID PTS.           ZMID(1) TO         ZMID(1) TO         B(2)
C                        ZMID(NRCALL)       ZMID(NRCALL)
C
C
C     IN THE ROLL CALL PHASE, SUMMING TAKES PLACE OVER THE LEGISLATORS  
C     AND ONLY THE PARAMETERS OF ONE ROLL CALL AT A TIME ARE UPDATED 
C     AND THESE ARE HELD IN B(1) AND B(2).  LIKEWISE, IN THE LEGISLATOR
C     PHASE SUMMING TAKES PLACE OVER THE ROLL CALLS AND ONLY ONE LEGISLATOR
C     COORDINATE (STORED IN B(1)) IS UPDATED AT A TIME.  IN THE UTILITY PHASE
C     SUMMATION IS OVER ALL THE DATA AND THE ESTIMATE OF BETA (THE SIGNAL TO 
C     NOISE RATIO) IS HELD IN B(1).     
C
C
      DIMENSION ALP(2),BTA(2),RR(2),KDATA(1601),
     CLL(2,2),LMO(513)
      character*1 kname
      CHARACTER*64 TITLE,FMT,FMT2
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /LUCK/ YBIGL(1601),YYBIGL(1601)
      COMMON /ROLL/ KPJP(1601,4)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /SAVE/ XSAVE(513,8,2),ZSAVE(1601,8,2),CSAVE(1601,8)
      COMMON /TCHOIC/ KPTSUM
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      COMMON /TOL/ TOLB
      COMMON /BOOK/ BBB(50)
      COMMON /VOTES/ KAV(1601),kay(1601),kan(1601),kname(513,25),jne
      COMMON/ INTGER /IIII
      COMMON /BAD/KBAD1,KBAD2,SPRM,SPRSD
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
C     OPEN(3,FILE='SEN97.DTA',STATUS='OLD')
C      OPEN(3,FILE='HOU49.DTA',STATUS='OLD')
      OPEN(3,FILE='SEN101.DTA',STATUS='OLD')
C      OPEN(3,FILE='G88Z1.DAT',STATUS='OLD')
      OPEN(4,FILE='NOMSTART.DAT',STATUS='OLD')
      OPEN(21,FILE='NOM21.DAT')
      OPEN(20,FILE='NOM20.DAT',form='binary')
      OPEN(23,FILE='NOM23.DAT')
      OPEN(31,FILE='NOM31.DAT')
      OPEN(33,FILE='NOM33.DAT')
      DATA ISENS/513*1/
  100 FORMAT(2I5,10F10.4)
  101 FORMAT(10X,10F10.4)
 1000 format(22x,4(i2.2,'.'))
 1001 FORMAT(8I5)
 1002 FORMAT(30X,3F14.4)
 1003 FORMAT(15F8.4)
 1004 FORMAT(20A4)
 1005 FORMAT(1X,19A4)
 1006 FORMAT(F8.4,I5)
 1007 FORMAT(3A4)
 1008 FORMAT(' DATA     ',I6,I7)
 1009 FORMAT(' CLEAN    ',I6,I7)
 1010 FORMAT(' BLACK BOX',I6,I7)
 1011 FORMAT(' RECODE   ',I6,I7)
 1019 FORMAT(24X,10F10.4)
 1029 FORMAT(7X,'CONSTRAINED',7X,'UNCONSTRAINED'/1X,'OFF OPPOSITE',
     C3X,'OFF SAME'/4X,I3,11X,I3,11X,I3)
 1036 FORMAT(6I1)
 1037 FORMAT(6I2)
 1111 format(' ELAPSED TIME OF JOB  ',4(i2.2,'.'))
 2001 FORMAT(3I5,9F7.3)
C
C
      call gettim(itim1,itim2,itim3,itim4)
      msec=itim1*60*60+itim2*60+itim3
C
C  CPUTIM RETURNS THE TIME IN SECONDS SINCE THE BEGINNING OF THE PROGRAM
C
      call CPUTIM(KTIM1)
      JIO=21
      KIO=23
      NITR=0
      JAN=0
C
C  READ IN PARAMETERS:  JANLST = # OF UNIVERSAL ITERATIONS
C                       NRCALL = # OF ROLL CALLS READ IN FROM DISK
C                       KLSEN  = INTEGER POSITION OF FURTHEST LEFT LEGISLATOR
C                                ALLOWS SETTING LIBERAL TO NEG VALUES
C                       MAXIT  = MAXIMUM NUMBER OF ITERATIONS IN BHHH 
C                                ALGORITHM (ESSENTIALLY NUMBER OF INVERSIONS
C                                OF INFORMATION MATRIX)
C                       MAXSQZ = NUMBER OF FUNCTION EVALUATIONS IN BHHH
C                                ALGORITHM.  BECAUSE OF STEPPING, THERE ARE
C                                MORE FUNCTION EVALUATIONS THAN THERE ARE
C                                ITERATIONS
C                       NS     = NUMBER OF DIMENSIONS TO BE ESTIMATED
C                       NQ     = NUMBER OF CHOICES (2 FOR ROLL CALL MODEL)
C                       JNE    = NUMBER OF CHARACTERS IN READ OF ROLL CALL
C                                MATRIX FORMAT STATEMENT.  THIS NUMBER WILL
C                                MATCH THE NUMBER OF CHARACTERS WRITTEN OUT
C                                IN FORMAT STATEMENT FOR WRITING OUT LEGIS-
C                                LATOR COORDINATES (SEE BELOW)
C
C  ******** EXAMPLE OF CONTROL CARD INPUT ********
C
C          NOMINAL UNIDIMENSIONAL UNFOLDING OF 1981-82 SENATE
C              3  966    3
C             30   15    1    2   19
C           15.0000  0.5000  0.40
C            0.0500   10
C          (4X,2A1,3X,A1,2X,5A1,2X,11A1,966I1)
C          (1X,I4,5X,5A1,4X,2A1,2X,A1,3X,11A1,1X,3F7.3,4I4)
C  ***********************************************
C
      READ(4,'(A)')TITLE
      READ(4,1001)JANLST,NRCALL,KLSEN
      READ(4,1001)MAXIT,MAXSQZ,NS,NQ,jne
      WRITE(JIO,'(A)')TITLE
      WRITE(JIO,1001)JANLST,NRCALL,KLSEN
      WRITE(JIO,1001)MAXIT,MAXSQZ,NS,NQ,jne
      WRITE(*,'(A)')TITLE
      WRITE(*,1001)JANLST,NRCALL,KLSEN
      WRITE(*,1001)MAXIT,MAXSQZ,NS,NQ,jne
      WRITE(KIO,'(A)')TITLE
C
C  READ IN STARTING VALUE FOR BETA AND VALUE FOR WEIGHT FOR ESTIMATING
C  THE UTILITY FUNCTION. THE LEGISLATOR AND ROLL CALL STARTING VALUES ARE
C  GENERATED INTERNALLY IN THE PROGRAM.
C
      READ(4,1003)(BBB(J),J=1,2),TOLB
      WRITE(JIO,1003)(BBB(J),J=1,2),TOLB
      WRITE(*,1003)(BBB(J),J=1,2),TOLB
C
C  XVMIN IS THE CUTOFF VALUE FOR MINORITY PERCENTAGE OF THE ROLL CALL (USUALLY
C    .05 OR .025) AND KVMIN IS THE MINIMUM NUMBER OF VOTES FOR A LEGISLATOR
C    TO BE INCLUDED IN THE ANALYSIS
C
      READ(4,1006)XVMIN,KVMIN
      WRITE(JIO,1006)XVMIN,KVMIN
      WRITE(*,1006)XVMIN,KVMIN
C
C  READ IN FORMAT OF ROLL CALL MATRIX (SEE EXAMPLE ABOVE)
C
      READ(4,'(A)')FMT
      WRITE(*,'(A)')FMT
      WRITE(JIO,'(A)')FMT
C
C  READ IN FORMAT TO WRITE LEGISLATOR COORDINATES TO DISK (SEE EXAMPLE ABOVE)
C
      READ(4,'(A)')FMT2
      WRITE(*,'(A)')FMT2
      WRITE(JIO,'(A)')FMT2
      KPTSUM=0
      NOB=0
      NBB=0
C
C  READ IN ROLL CALL DATA FROM UNIT 3 USING FORMAT STATEMENT READ IN
C     ABOVE.  NOTE THAT JNE = # A1'S IN FORMAT STATEMENT
C
C     KNAME DESCRIBES NAME, PARTY, STATE ETC. OF THE LEGISLATOR
C     KDATA IS ROLL CALL DATA.
C
  10  READ(3,FMT,END=20)(KNAME(NOB+1,J),J=1,jne),(KDATA(J),J=1,NRCALL)
      NBB=NBB+1
      ISENS(NBB)=1
      NOB=NOB+1
C
C  WRITE ROLL CALL DATA TO BINARY FILE.  THIS FILE IS RE-PROCESSED IN 
C    SUBROUTINE CLEAN
C
      WRITE(20)(KDATA(J),J=1,NRCALL)
      GO TO 10
  20  NOBS=NOB
      call CPUTIM(KTIM1)
      WRITE(JIO,1008)KTIM1,KTIM1
      WRITE(*,1008)KTIM1,KTIM1
      LTIME=KTIM1      
C
C  SUBROUTINE CLEAN--THROWS OUT ALL VOTES WITH LESS THAN XVMIN MAJ. AND
C    ALL LEGISLATORS VOTING LESS THAN KVMIN TIMES
C
      call CLEAN(XVMIN,KVMIN)
      CALL CPUTIM(KTIM1)
      MTIME=KTIM1-LTIME
      WRITE(JIO,1009)KTIM1,MTIME
      WRITE(*,1009)KTIM1,MTIME
      LTIME=KTIM1
C
C  SUBROUTINE BLACKB--GENERATES STARTING VALUES FOR LEGISLATORS BY 
C    PERFORMING A PSEUDO-SINGULAR VALUE DECOMPOSITION OF NY ROLL CALLS
C
      NY=100
C      NS=5
      call BLACKB(NOBS,NY,NS,KVMIN,KIO,NRCALL)
      CALL CPUTIM(KTIM1)
      MTIME=KTIM1-LTIME
      WRITE(JIO,1010)KTIM1,MTIME
      WRITE(*,1010)KTIM1,MTIME
      LTIME=KTIM1
C
C  SUBROUTINE RECODE--GENERATES STARTING VALUES FOR ROLL CALL MIDPOINTS
C    AND DETERMINES POLARITY OF ROLL CALL
C
      call RECODE(NOBS,NRCALL,KLSEN,KIO)
      CALL CPUTIM(KTIM1)
      MTIME=KTIM1-LTIME
      WRITE(JIO,1011)KTIM1,MTIME
      WRITE(*,1011)KTIM1,MTIME
      LTIME=KTIM1
      NTIME(1)=LTIME
C
C  **UNIVERSAL LOOP**
C     3-STEP ESTIMATION; BETA, ROLL CALLS, LEGISLATORS
C     JANLST IS # OF UNIVERSAL ITERATIONS.
C
C
      KKKK=0
      DO 9999 IIII=1,JANLST
      JAN=JAN+1
      DO 6666 NSTEP=0,2
C
C     UTILITY FUNCTION PHASE--BETA ESTIMATED 
C
      IF(NSTEP.EQ.0.AND.IIII.GT.1)THEN
           NOPAR=1
           NITR=NITR+1
           call MAXLIK
      ENDIF
C
C     ROLL call PHASE--MIDPOINT AND DISTANCE BETWEEN TWO OUTCOMES ESTIMATED
C
      IF(NSTEP.EQ.1)THEN
           NOPAR=2
           NITR=NITR+1
           call MAXLIK
           call OUTWRT(NRCALL,NSTEP,JIO,KIO,JAN,9999,KPJP,YBIGL,FMT2)
      ENDIF
C
C     LEGISLATOR PHASE--COORDINATE ESTIMATED
C
      IF(NSTEP.EQ.2)THEN
           NOPAR=1
           NITR=NITR+1
           call MAXLIK
           call OUTWRT(NOBS,NSTEP,JIO,KIO,JAN,9999,KPJP,YYBIGL,FMT2)
      ENDIF
 6666 CONTINUE
C
C  CALCULATE NUMBER OF CONSTRAINED ROLL CALLS AND CORRELATION BETWEEN 
C  CURRENT PARAMETER ESTIMATES AND THOSE FROM PREVIOUS ITERATIONS
C
      KRSCLL=NRCALL-KBAD1-KBAD2
      call CORR(ALP,BTA,RR,0)
      WRITE(KIO,2001)KBAD2,KBAD1,KRSCLL,SPRM,SPRSD,ALP(1),BTA(1),RR(1),
     CSSS(1),SSS(3),sss(4),SSS(2)
C      WRITE(*,2001)KBAD2,KBAD1,KRSCLL,SPRM,SPRSD,ALP(1),BTA(1),RR(1),
C     CSSS(1),SSS(3),sss(4),SSS(2)
 9999 CONTINUE
C
C  END OF UNIVERSAL LOOP
C    SETUP PARAMETERS FOR FINAL ESTIMATE OF BETA AND ROLL CALLS USING
C    FINAL ADJUSTED LEGISLATOR COORDINATES
C
C  FINAL ESTIMATION OF BETA
C
      JAN=JAN+1
      JANLST=1
      NSTEP=0
      NOPAR=1
      NITR=NITR+1
      call MAXLIK
C
C  FINAL ESTIMATION OF ROLL CALL PARAMETERS
C
      NSTEP=1
      NOPAR=2
      NITR=NITR+1
      call MAXLIK
      call OUTWRT(NRCALL,NSTEP,JIO,KIO,JAN,9999,KPJP,YBIGL,FMT2)
C
C  SETUP PARAMETERS TO CALCULATE CLASSIFICATIONS FOR LEGISLATORS
C
      NSTEP=2
      JANLST=JAN
      DO 3 II=1,NOBS
      LMO(II)=II
      JJ=II
      B(1)=XDATA(II)
C
C     SUBROUTINE CROSS COMPUTES 2x2 PREDICTIONS FOR ROLL CALLS AND LEGS.
C
      call cross(JJ,LL)
      JJ=1
      DO 4 I=1,2
      DO 4 J=1,2
      KPJP(II,JJ)=LL(I,J)
  4   JJ=JJ+1
  3   CONTINUE
C
C  WRITE OUT FINAL ESTIMATE OF LEGISLATORS WITH CROSS CLASSIFICATIONS
C
      call OUTWRT(NOBS,NSTEP,KIO,KIO,999,999,KPJP,YYBIGL,FMT2)
C
C  SETUP PARAMETERS TO CALCULATE CLASSIFICATIONS FOR ROLL CALLS
C
      NSTEP=1
      DO 5 II=1,NRCALL
      JJ=II
      B(1)=DYN(II)
      B(2)=ZMID(II)
      call cross(JJ,LL)
      JJ=1
      DO 6 I=1,2
      DO 6 J=1,2
      KPJP(II,JJ)=LL(I,J)
  6   JJ=JJ+1
  5   CONTINUE
C
C  WRITE OUT FINAL ESTIMATES OF ROLL CALLS WITH CROSS CLASSIFICATIONS
C
      call OUTWRT(NRCALL,NSTEP,KIO,KIO,999,999,KPJP,YBIGL,FMT2)
C
C  WRITE OUT FINAL NUMBERS OF CONSTRAINED ROLL CALLS
C
      KRSCLL=NRCALL-KBAD1-KBAD2
      WRITE(JIO,1029)KBAD2,KBAD1,KRSCLL
      WRITE(*,1029)KBAD2,KBAD1,KRSCLL
C
C  SORT LEGISLATORS AND WRITE OUT THE 6 MOST EXTREME LEGISLATORS
C
      call RSORT(XDATA,NOBS,LMO)
      WRITE(JIO,1039)
      WRITE(*,1039)
 1039 FORMAT(' SAG IN RECOVERED LEGISLATOR COORDINATES')
      DO 102 I=1,NOBS
C
C  WRITE OUT PARTIAL LIST OF LEGISLATOR COORDINATES--NOTE THAT
C    THIS FORMAT IS UNIQUE TO THE ROLL CALL DATA -- FOR DIFFERENT
C    MATRICES, THIS CODE WILL HAVE TO BE COMMENTED OUT -- SEE
C    EXAMPLE BELOW
C
      IF(I.GT.3.AND.I.LT.NOBS-2)GO TO 102
      WRITE(JIO,FMT2)ISENS(LMO(I)),(KNAME(LMO(I),J),J=4,8),
     Ckname(LMO(i),1),kname(LMO(i),2),kname(LMO(i),3),
     C(kname(LMO(i),j),j=9,JNE),xdata(i)
      WRITE(*,FMT2)ISENS(LMO(I)),(KNAME(LMO(I),J),J=4,8),
     Ckname(LMO(i),1),kname(LMO(i),2),kname(LMO(i),3),
     C(kname(LMO(i),j),j=9,JNE),xdata(i)
C      WRITE(JIO,FMT2)ISENS(LMO(I)),(KNAME(LMO(I),J),J=1,JNE),
C     C     XDATA(LMO(I))
C      WRITE(*,FMT2)ISENS(LMO(I)),(KNAME(LMO(I),J),J=1,JNE),
C     C     XDATA(LMO(I))
  102 CONTINUE
      call gettim(jtim1,jtim2,jtim3,jtim4)
      write(JIO,1000)itim1,itim2,itim3,itim4
      write(JIO,1111)jtim1,jtim2,jtim3,jtim4
      write(*,1000)itim1,itim2,itim3,itim4
      write(*,1111)jtim1,jtim2,jtim3,jtim4
      STOP
      END
C
C  **************************************************************************
C    SUBROUTINE MAXLIK---CALLED BY MAIN.  INITIALIZES VARIABLES FOR 
C          ESTIMATION OF PARAMETERS
C  **************************************************************************
C
      SUBROUTINE MAXLIK
      DIMENSION LL(2,2),MM(2,2)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /ROLL/KPJP(1601,4)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
      COMMON /BOOK/ BBB(50)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      EXTERNAL FUNNEL
  23  FORMAT(' TOTAL VOTES PREDICTED FOR',I5,1X,'ROLL-CALLS'/13X,
     C'ACTUAL'/7X,'  YEA     NAY  ')
  24  FORMAT(' TOTAL VOTES PREDICTED FOR',I5,1X,'LEGISLATORS'/13X,
     C'ACTUAL'/7X,'  YEA     NAY  ')
  25  FORMAT('   YEA  ',I6,2X,I6/'   NAY  ',I6,2X,I6)
C
C  ***** UTILITY FUNCTION PHASE *****
C
      IF(NSTEP.EQ.0)THEN
           B(1)=BBB(1)
C
C  CALL SUBROUTINE NORMZ TO NORMALIZE COORDINATES AND CHECK FOR SAG
C    IN LEGISLATOR COORDINATES
C
           call NORMZ
           NFEVAL=0
           call BHHH(FUNNEL,F)
           call RPRINT(F,B,V)
      ENDIF
C
C  ***** ROLL CALL PHASE *****
C
      IF(NSTEP.EQ.1)THEN
           DO 22 I=1,2
           DO 22 J=1,2
  22       MM(I,J)=0
           NEQ=0
  1        CONTINUE
           NEQ=NEQ+1
           NFEVAL=0
           B(1)=DYN(NEQ)
           B(2)=ZMID(NEQ)
C           IF(ZMID(NEQ).LT.DYN(NEQ))DYN(NEQ)=DYN(NEQ)-.5
           call BHHH(FUNNEL,F)
           call RPRINT(F,B,V)
           call cross(NEQ,LL)
           JJ=1
           DO 21 I=1,2
           DO 21 J=1,2
           KPJP(NEQ,JJ)=LL(I,J)
           JJ=JJ+1
  21       MM(I,J)=MM(I,J)+LL(I,J)
           IF(NEQ.LT.NRCALL)GO TO 1
           WRITE(KIO,23)NRCALL
           WRITE(KIO,25)((MM(I,J),J=1,2),I=1,2)
      ENDIF
C
C  ***** LEGISLATOR PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           DO 26 I=1,2
           DO 26 J=1,2
  26       MM(I,J)=0
           NEQ=0
  10       CONTINUE
           NEQ=NEQ+1
           NFEVAL=0
           B(1)=XDATA(NEQ)
           call BHHH(FUNNEL,F)
           call RPRINT(F,B,V)
           call cross(NEQ,LL)
           JJ=1
           DO 27 I=1,2
           DO 27 J=1,2
           KPJP(NEQ,JJ)=LL(I,J)
           JJ=JJ+1
  27       MM(I,J)=MM(I,J)+LL(I,J)
           IF(NEQ.LT.NOBS)GO TO 10
           WRITE(KIO,24)NOBS
           WRITE(KIO,25)((MM(I,J),J=1,2),I=1,2)
      ENDIF
      RETURN
      END
C
C  **************************************************************************
C    SUBROUTINE BHHH--CALLED BY MAXLIK.  IMPLEMENTS THE BHHH ALGORITHM FOR  
C       MAXIMIZING LOG-LIKELIHOOD
C  **************************************************************************
C
      SUBROUTINE BHHH(FUNNEL,XLNL)
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /LIKE/ SVLNL,FNLNL,ICNVG,ITT,NVAL
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /TOL/ TOLB
      DIMENSION AV(25,25),KCONST(20),
     czmat(25,25),wvec(25),fv1(25),fv2(25)
      EXTERNAL FUNNEL
  101 format(' PERFORMANCE INDEX EIGENVECTOR/VALUE ROUTINE=',I4)
C
      ICNVG=0
      DO 50 I=1,NOPAR
      KCONST(I)=1
      DELTAB(I)=0.0
   50 TESTB(I)=B(I)
C
C          BERNDT, HALL, HALL, AND HAUSMAN METHOD
C
      DO 1000 IT=1,MAXIT
      IFDRV=1
c
C      
      XLNL=FUNNEL(0.0)
C
      IF(IT.EQ.1)SVLNL=-XLNL
      IFDRV=0
      IF(NOPAR.EQ.1)GO TO 20
      IF(NSTEP.EQ.0) KCONST(1)=1
      IF(NSTEP.EQ.1) KCONST(1)=1
      IF(NSTEP.EQ.1) KCONST(2)=1
      IF(NSTEP.EQ.1) KCONST(3)=0
      MSUM=KCONST(1)+KCONST(2)+KCONST(3)
      IF(MSUM.EQ.0)GO TO 1200
      IF(MSUM.EQ.1)GO TO 21
      IF(MSUM.EQ.2)GO TO 23
C     
C         V11 V12 V13
C     V=  V21 V22 V23
C         V31 V32 V33
C
C     AV = INVERSE-V
C
C  CALCULATE MATRIX INVERSE
C
      call rs(25,3,v,wvec,1,ZMAT,fv1,fv2,ier)
      DO 60 I=1,3
      DO 60 KX=1,3
      SUM=0.0
      DO 61 J=1,3
  61  SUM=SUM+ZMAT(KX,J)*(1.0/WVEC(J))*ZMAT(I,J)
  60  av(I,KX)=SUM
      DET=V(1,1)*V(2,2)*V(3,3)-V(1,1)*V(2,3)*V(3,2)+V(1,2)*V(2,3)*V(3,1)
      DET=DET-V(1,2)*V(2,1)*V(3,3)+V(1,3)*V(2,1)*V(3,2)-V(1,3)*V(2,2)*
     CV(3,1)
      IF(ABS(DET).LE..00000001)ICNVG=9
      IF(ABS(DET).LE..00000001)RETURN
      DELTAB(1)=(AV(1,1)*GRAD(1)+AV(2,1)*GRAD(2)+AV(3,1)*GRAD(3))
      DELTAB(2)=(AV(1,2)*GRAD(1)+AV(2,2)*GRAD(2)+AV(3,2)*GRAD(3))
      DELTAB(3)=(AV(1,3)*GRAD(1)+AV(2,3)*GRAD(2)+AV(3,3)*GRAD(3))
      DO 77 I=1,3
      DO 77 J=1,3
      V(I,J)=AV(J,I)
77    CONTINUE
      GO TO 22
C
C     WHEN ONE OF THE THREE KCONST'S IS 1.
C
21    IF(KCONST(1).EQ.1.AND.KCONST(2).EQ.0.AND.KCONST(3).EQ.0)GO TO 30
      IF(KCONST(1).EQ.0.AND.KCONST(2).EQ.1.AND.KCONST(3).EQ.0)GO TO 31
      IF(KCONST(1).EQ.0.AND.KCONST(2).EQ.0.AND.KCONST(3).EQ.1)GO TO 32
30    IC=1
      JC=2
      KC=3
      GO TO 35
31    IC=2
      JC=1
      KC=3
      GO TO 35
32    IC=3
      JC=1
      KC=2
35    IF(ABS(V(IC,IC)).LE..00000001)ICNVG=9
      IF(ABS(V(IC,IC)).LE..00000001)RETURN
      DELTAB(IC)=GRAD(IC)/V(IC,IC)
      V(IC,IC)=1.0/V(IC,IC)
      DELTAB(JC)=0.0
      DELTAB(KC)=0.0
      GO TO 22
C
C     WHEN TWO OF THE THREE KCONST'S ARE 1'S.
C
23    IF(KCONST(1).EQ.1.AND.KCONST(2).EQ.1.AND.KCONST(3).EQ.0)GO TO 40
      IF(KCONST(1).EQ.1.AND.KCONST(2).EQ.0.AND.KCONST(3).EQ.1)GO TO 41
      IF(KCONST(1).EQ.0.AND.KCONST(2).EQ.1.AND.KCONST(3).EQ.1)GO TO 42
40    IC=1
      JC=2
      KC=3
      GO TO 45
41    IC=1
      JC=3
      KC=2
      GO TO 45
42    IC=2
      JC=3
      KC=1
45    DET=V(IC,IC)*V(JC,JC)-V(IC,JC)*V(JC,IC)
      IF(ABS(DET).LE..00000001)ICNVG=9
      IF(ABS(DET).LE..00000001)RETURN
      DELTAB(IC)=(V(JC,JC)*GRAD(IC)-V(IC,JC)*GRAD(JC))/DET
      DELTAB(JC)=(V(IC,IC)*GRAD(JC)-V(JC,IC)*GRAD(IC))/DET
      DELTAB(KC)=0.0
      S11=V(IC,IC)
      V(IC,IC)=V(JC,JC)/DET
      V(JC,JC)=S11/DET
      V(IC,JC)=-V(IC,JC)/DET
      V(JC,IC)=-V(JC,IC)/DET
      GO TO 22
  20  IF(ABS(V(1,1)).LE..00000001)ICNVG=9
      IF(ABS(V(1,1)).LE..00000001)RETURN
      DELTAB(1)=GRAD(1)/V(1,1)
      V(1,1)=1.0/V(1,1)
      DELTAB(2)=0.0
      DELTAB(3)=0.0
      GO TO 22
  22  CONTINUE
      COSDG=DELTAB(1)*GRAD(1)+DELTAB(2)*GRAD(2)+DELTAB(3)*
     CGRAD(3)
C
C  AFTER 3 ITERATIONS STEPSIZE DEFAULTS TO PREVIOUS STEPSIZE
C
      IF(IT.LE.3)ALAMB=1.0
      IFBETR=1
      CALL STEPR(XLNL,XLNLNW,ALAMB,IFBETR,MAXSQZ,ISQZ,FUNNEL)
C
C          PRINT ITERATION RESULTS
C
      IF(NSTEP.EQ.0)WRITE(KIO,1918)IT,(TESTB(J),J=1,NOPAR),
     CALAMB,COSDG,XLNL,XLNLNW
 1918 FORMAT(I5,12F10.4/5X,12F10.4)
C
C          CHECK FOR CONVERGENCE OF LIKELIHOOD FUNCTION.
C
C
      IF(COSDG.LT.(TOLB**2))GO TO 1200
      IF(IFBETR.EQ.0)GO TO 1500
      DO 710 I=1,NOPAR
  710 B(I)=TESTB(I)
 1000 CONTINUE
      GO TO 1100
C
C          FAILURE TO CONVERGE
C
 1100 ICNVG=1
      ITT=IT
      GO TO 1206
C
C          CONVERGENCE
C
 1200 ICNVG=0
      ITT=IT
 1206 CONTINUE
 1207 DO 705 I=1,NOPAR
  705 B(I)=TESTB(I)
      XLNL=-FUNNEL(0.0)
      FNLNL=XLNL
      ITT=IT
      NVAL=NFEVAL
      RETURN
C
C          FAILURE TO IMPROVE LNL
C
 1500 ICNVG=2
      DO 650 I=1,NOPAR
  650 TESTB(I)=B(I)
      GO TO 1207
      END
C
C  ***************************************************************************
C    SUBROUTINE STEPR---CALLED BY BHHH.  CALCULATES STEPSIZE FOR GRADIENT
C  ***************************************************************************
C
      SUBROUTINE STEPR(F,F0,R,IFOK,MAXSQZ,NSQZ,FUNNEL)
C
C
      EXTERNAL FUNNEL
C
C
C        IF STEPPING FORWARD REDUCES -LNL, DOUBLE THE STEP.
C        IF NOT, HALVE THE STEP.  ONCE STEPPING TAKES PLACE 
C        IN ONE DIRECTION, IT CONTINUES IN THAT DIRECTION
C        UNTIL NO FURTHER IMPROVEMENT OCCURS.
C
C
C
C  THIS SECTION INTIALIZES LOG-LIKELIHOOD--IF IT IS BETTER THAN OLD
C    VALUE THE STEP SIZE IS DOUBLED IN CODE SEGMENT BELOW.  IF IT IS
C    WORSE, THE STEP SIZE IN HALVED IN CODE SEGMENT IMMEDIATELY BELOW
C
      IFB = IFOK
      R0 = R
      NSQZ = 0
      IFOK=1
    2 F0 = FUNNEL(R0)
      IF(F0.LT.F.AND.IFB.EQ.1)GO TO 10
  99  R0=R0/2.
      NSQZ = NSQZ +1
      IF(NSQZ .LE. MAXSQZ) GO TO 2
      IFOK=0
      R = R0
      RETURN
   10 IF(NSQZ .GT. 0) GO TO 13
C
C  IF IMPROVEMENT, STEP SIZE IS DOUBLED
C
   14 R1 = R0 * 2.
      F1 = FUNNEL(R1)
      NSQZ = NSQZ +1
      IF(F1 .GT. F0) GO TO 11
      IF(F1.LT.0.0)GO TO 11
      R0 = R1
      F0 = F1
      IF(NSQZ .LT. MAXSQZ) GO TO 14
      R = R1
      F0 = F1
      RETURN
C
   11 F0 = FUNNEL(R0)
   13 R = R0
      RETURN
      END
C
C  **************************************************************************
C    FUNCTION FUNNEL---CALLED BY STEPR AND BY BHHH WHEN INITIALIZING.  IT
C      CALCULATES THE UPDATED PARAMETER VECTOR AS NOTED BELOW AND CALLS
C      SUBROUTINE LOGLIK WHICH CALCULATES THE CORRESPONDING LOG-LIKELIHOOD
C  **************************************************************************
C
      REAL FUNCTION FUNNEL(XLBA)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
C
C          UPDATE TRIAL PARAMETER VECTOR.
C
C        TESTB = B + XLBA * DELTAB
C
C           WHERE DELTAB = QMAT(INVERSE) * G
C           AND XLBA = STEPSIZE
C
      DO 10 I=1,NOPAR
  10  TESTB(I)=B(I)+XLBA*DELTAB(I)
      call LOGLIK(TESTB,FLIKE,GRAD,V)
      FUNNEL=FLIKE
      RETURN
      END
C
C  ***************************************************************************
C    SUBROUTINE LOGLIK---CALLED BY FUNNEL.  CALCULATES LOG-LIKELIHOOD.  FOR
C       EACH APPROPRIATE OBSERVATION, IT CALLS SUBROUTINE ITHOBS (ITH 
C       OBSERVATION) WHICH RETURNS THE LOG-LIKELIHOOD.  THE SUM IS CUMULATED
C       BELOW.
C  ***************************************************************************
C
      SUBROUTINE LOGLIK(B,FLIKE,GRAD,V)
C
C
      DIMENSION B(50),GRAD(50),V(25,25),G(50),XBIGL(1601)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /LUCK/ YBIGL(1601),YYBIGL(1601)
C
C
      IF(NSTEP.EQ.0)NNBS=NOBS
      IF(NSTEP.EQ.1)NNBS=NOBS
      IF(NSTEP.EQ.2)NNBS=NRCALL
      NFEVAL=NFEVAL+1
      FLIKE=0.0
      IF(NSTEP.EQ.2)YYBIGL(NEQ)=0.0
      IF(NSTEP.NE.1)GO TO 3
      DO 2 J=NEQ,NRCALL
  2   YBIGL(J)=0.0
  3   CONTINUE
      IF(IFDRV.EQ.0)GO TO 202
      DO 201 J=1,NOPAR
      GRAD(J)=0.0
  201 CONTINUE
      DO 101 J=1,NOPAR
      DO 101 I=1,NOPAR
  101 V(I,J)=0.0
  202 CONTINUE
C
C          LOOP OVER NUMBER OF OBSERVATIONS NNBS AND SUM LOG LIKELIHOOD
C     AND ITS GRADIENT WITH RESPECT TO B.
C
      I=0
  200 I=I+1
      IF(I.GT.NNBS)GO TO 211
      call ITHOBS(I,B,XLNL,G,XBIGL)
      IF(NSTEP.EQ.2)YYBIGL(NEQ)=YYBIGL(NEQ)+XLNL
      IF(NSTEP.EQ.1)YBIGL(NEQ)=YBIGL(NEQ)+XLNL
      XLNL=-XLNL
      FLIKE=FLIKE+XLNL
      IF(IFDRV.EQ.0)GO TO 200
      DO 205 J=1,NOPAR
  205 GRAD(J)=GRAD(J)+G(J)
C
C          EVALUATE QMAT, THE APPROXIMATION TO THE HESSIAN, AS THE
C     OUTER PRODUCT OF THE GRADIENT VECTOR ONLY FOR THE BHHH METHOD.
C
      DO 206 L=1,NOPAR
      DO 206 J=1,NOPAR
  206 V(J,L)=V(J,L)+G(J)*G(L)
      GO TO 200
  211 RETURN
      END
C
C  ***************************************************************************
C    SUBROUTINE ITHOBS--CALLED BY LOGLIK.  CALCULATES THE LOG-LIKELIHOOD FOR
C      EACH OBSERVATION (I.E. THE LOG-LIKELIHOOD FOR ONE COLUMN/ROW IN THE
C      INPUT CHOICE MATRIX.
C  ***************************************************************************
C
      SUBROUTINE ITHOBS(JT,B,XL,G,XBIGL)
      DIMENSION DSQ(4,1601),XB(4,1601),
     CEXB(4,1601),DLDX(4),B(50),G(50),
     CPHI(1601),XBIGL(1601)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      COMMON /BOOK/ BBB(50)
C
C   SUBROUTINE TO ESTIMATE MULTINOMIAL LOGIT FOR SPATIAL MODELS
C
C   NP=NUMBER OF INDIVIDUALS=NOBS
C   NS=NUMBER OF DIMENSIONS
C   NQ=NUMBER OF STIMULI
C
C
C   B (PARAMETER VECTOR IN MAXLIK) SHOULD HAVE THE FOLLOWING
C   FORMAT:
C
C    IF NSTEP EQUALS 0
C     B(1) IS THE SIGNAL TO NOISE RATIO, BETA
C
C
C    IF NSTEP EQUALS 1
C     B(1) IS THE DISTANCE/2 BETWEEN THE TWO OUTCOMES
C     B(2) IS THE MIDPOINT OF THE OUTCOMES
C
C    IF NSTEP EQUALS 2
C     B(1) IS THE ITH LEGISLATOR'S COORDINATE
C
C
C    LDATA(JT, ... )IS THE CHOICE VARIABLE
C        (E.G. YES=1, NO=2)
C    XDATA(JT) ARE THE INDIVIDUAL COORDINATES
C
C
C
      IF(JT.NE.1)GO TO 11
      IF(NSTEP.EQ.0)THEN
         KRCALL=1
         LRCALL=NRCALL
         BETA=B(1)
         WEIGHT=BBB(2)
      ENDIF
      IF(NSTEP.NE.0)THEN
         BETA=BBB(1)
         WEIGHT=BBB(2)
      ENDIF
  11  CONTINUE
      DO 27 JJ=1,5
  27  G(JJ)=0.0
      XL=0.0
C
C  ***** LEGISLATOR PHASE *****
C    COMPUTE DERIVATIVE FOR LEGISLATORS
C
C
      IF(NSTEP.EQ.2)THEN
         WTSQ=WEIGHT**2
         ICH=LDATA(NEQ,JT)
         IF(ICH.LE.0)RETURN
         D2Y=(ABS(B(1)-ZMID(JT)+DYN(JT)))**2
         D2N=(ABS(B(1)-ZMID(JT)-DYN(JT)))**2
         XBY=-D2Y*WTSQ/2.0
         XBY=BETA*EXP(XBY)
         EXBY=EXP(XBY)
         XBN=-D2N*WTSQ/2.0
         XBN=BETA*EXP(XBN)
         EXBN=EXP(XBN)
         PHIXI=EXBY+EXBN
         AA=ALOG(PHIXI)
         XL=-AA
         IF(ICH.EQ.1)XL=XL+XBY
         IF(ICH.EQ.2)XL=XL+XBN
         DLDX(1)=B(1)-ZMID(JT)+DYN(JT)
         DLDX(2)=B(1)-ZMID(JT)-DYN(JT)
         DLDX(1)=DLDX(1)*WTSQ+WEIGHT*(DLDX(1)**2)
         DLDX(2)=DLDX(2)*WTSQ+WEIGHT*(DLDX(2)**2)
         IF(ICH.EQ.1)G(1)=-DLDX(1)*XBY
         IF(ICH.EQ.2)G(1)=-DLDX(2)*XBN
         G(1)=G(1)+(DLDX(1)*XBY*EXBY+DLDX(2)*XBN*EXBN)/PHIXI
C
C  IF LEGISLATOR IS CONSTRAINED DUE TO SAG PROBLEM, SET DERIVATIVE TO ZERO
C
         g(1)=g(1)*isens(neq)
      ENDIF
C
C  ***** ROLL CALL PHASE *****
C  ROLL CALL DERIVATIVES
C
C
      IF(NSTEP.EQ.1)THEN
         XBIGL(NEQ)=0.0
         ICH=LDATA(JT,NEQ)
         IF(ICH.LE.0)RETURN
         DSQ(1,NEQ)=(WEIGHT*(XDATA(JT)-B(2)+B(1)))**2
         DSQ(2,NEQ)=(WEIGHT*(XDATA(JT)-B(2)-B(1)))**2
C
C   COMPUTE PHI
C
         Phi(NEQ)=0.0
         DO 6 I=1,NQ
         XB(I,NEQ)=-DSQ(I,NEQ)/(2.0)
         IF(XB(I,NEQ).LT.-10.0)GO TO 10
         XB(I,NEQ)=BETA*EXP(XB(I,NEQ))
  10     IF(XB(I,NEQ).LT.-10.0)XB(I,NEQ)=0.0
         EXB(I,NEQ)=EXP(XB(I,NEQ))
  6      PHI(NEQ)=PHI(NEQ)+EXB(I,NEQ)
C
C   COMPUTE LIKELIHOOD
C
         CC=ALOG(PHI(NEQ))
         XL=-CC
         XBIGL(NEQ)=-CC
         IF(ICH.GT.0)XL=XB(ICH,NEQ)+XL
         IF(ICH.GT.0)XBIGL(NEQ)=XBIGL(NEQ)+XB(ICH,NEQ)
C
C    COMPUTE DERIVATIVES FOR ROLL CALL OUTCOMES
C
         WTSQ=WEIGHT**2
         DLDX(1)=XDATA(JT)-B(2)+B(1)
         DLDX(2)=XDATA(JT)-B(2)-B(1)
         IF(ICH.EQ.1)G(1)=-WTSQ*DLDX(1)*XB(1,NEQ)
         IF(ICH.EQ.2)G(1)= WTSQ*DLDX(2)*XB(2,NEQ)
C         G(1)=WTSQ*(DLDX(2)*XB(2,NEQ)-DLDX(1)*XB(1,NEQ))
         G(1)=G(1)-WTSQ*(DLDX(2)*XB(2,NEQ)*EXB(2,NEQ) -
     C                   DLDX(1)*XB(1,NEQ)*EXB(1,NEQ))/PHI(NEQ)
         G(2)=WTSQ*DLDX(ICH)*XB(ICH,NEQ)
         G(2)=G(2)-WTSQ*(DLDX(1)*XB(1,NEQ)*EXB(1,NEQ)+
     C                   DLDX(2)*XB(2,NEQ)*EXB(2,NEQ))/PHI(NEQ)
      ENDIF
C
C  ***** UTILITY FUNCTION PHASE *****
C  DERIVATIVES FOR BETA
C
C
      IF(NSTEP.EQ.0)THEN
         DO 26 II=KRCALL,LRCALL
         XBIGL(II)=0.0
         ICH=LDATA(JT,II)
         IF(ICH.LE.0)GO TO 26
         DSQ(1,II)=(WEIGHT*(XDATA(JT)-ZMID(II)+DYN(II)))**2
         DSQ(2,II)=(WEIGHT*(XDATA(JT)-ZMID(II)-DYN(II)))**2
C
C   COMPUTE PHI
C
         PHI(II)=0.0
         DO 66 I=1,NQ
         XB(I,II)=-DSQ(I,II)/(2.0)
         IF(XB(I,II).LT.-10.0)GO TO 67
         XB(I,II)=BETA*EXP(XB(I,II))
  67     IF(XB(I,II).LT.-10.0)XB(I,II)=0.0
         EXB(I,II)=EXP(XB(I,II))
  66     PHI(II)=PHI(II)+EXB(I,II)
C
C   COMPUTE LIKELIHOOD
C
         CC=ALOG(PHI(II))
         XL=XL-CC
         XBIGL(II)=-CC
         IF(ICH.GT.0)XL=XB(ICH,II)+XL
         IF(ICH.GT.0)XBIGL(II)=XBIGL(II)+XB(ICH,II)
  26     CONTINUE
C
C   COMPUTE FIRST PARTIALS OF BETA
C
         DO 77 II=KRCALL,LRCALL
         ICH=LDATA(JT,II)
         IF(ICH.LE.0) GO TO 77
         DO 7 K=1,NQ
         T=-EXB(K,II)/PHI(II)
         IF(K.EQ.ICH)T=T+1.0
         G(1)=G(1)+T*(1.0/BETA)*XB(K,II)
  7      CONTINUE
  77     CONTINUE
      ENDIF
      RETURN
      END
C
C  **************************************************************************
C    SUBROUTINE RPRINT---CALLED BY MAXLIK.  STORES NEW PARAMETER ESTIMATES
C       AND (OPTIONALLY) WRITES PARAMETER ESTIMATES TO DISK.  IN ROLL CALL
C       PHASE, IF CONVERGED ESTIMATES ARE "UNREALISTIC", GRID2
C       SUBROUTINE IS CALLED TO PRODUCED "APPROPRIATE" CONSTRAINED 
C       ESTIMATES
C  **************************************************************************
C
      SUBROUTINE RPRINT(F,B,V)
C
C
      DIMENSION SSSE(25),TTTE(25),B(50),V(25,25),ALP(2),BTA(2),RR(2)
      character*1 kname
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      COMMON /SENMIN/ sss(100),KSMIN,KSMAX
      COMMON /BOOK/ BBB(50)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /SAVE/ XSAVE(513,8,2),ZSAVE(1601,8,2),CSAVE(1601,8)
      COMMON /LIKE/ SVLNL,FNLNL,ICNVG,ITT,NVAL
      COMMON /TCHOIC/ KPTSUM
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /STD/ STDDVX(513),STDDVZ(1601,3)
      COMMON /VOTES/ KAV(1601),KAY(1601),KAN(1601),KNAME(513,25),JNE
C
 1001 FORMAT(I5,2F13.4,3I4,4F9.4/34X,5F9.4/25X,6F9.4)
 1002 FORMAT(1X,2I4,2I7,F14.4,4F9.4)
 1003 FORMAT(1X,I4,1X,19A1,1X,2F13.4,3I4,4F9.4)
 1010 FORMAT(1X,I4,2F13.4,3I4,4F9.4/30X,F13.4,4F9.4)
 1011 FORMAT(I5,2F13.4,3I4,4F9.4/34X,5F9.4)
      DO 10 I=1,NOPAR
      SE=SQRT(ABS(V(I,I)))
      T=0.0
      IF(SE.GT.0) T=B(I)/SE
      SSSE(I)=SE
      TTTE(I)=T
   10 CONTINUE
C
C  ***** UTILITY FUNCTION PHASE *****
C
      IF(NSTEP.EQ.0)THEN
           WRITE(KIO,1011)NEQ,SVLNL,FNLNL,ITT,
     C     NVAL,ICNVG,BBB(1),B(1),SSSE(1),TTTE(1)
C           WRITE(*,1011)NEQ,SVLNL,FNLNL,ITT,
C     C     NVAL,ICNVG,BBB(1),B(1),SSSE(1),TTTE(1)
           call CPUTIM(KTIM1)
           IITIME=ktim1
           NTIME(NITR)=KTIM1
           JITIME=IITIME
           IF(NITR.GT.1)JITIME=NTIME(NITR)-NTIME(NITR-1)
C
C     KPTSUM=TOTAL # OF CHOICES.
C
           AA=F/FLOAT(KPTSUM)
C
C     AA=EXP(AA) IS GEOMETRIC MEAN PROBABILITY OF UTILITY FUNCTION.
C
           AA=EXP(AA)
           BBB(1)=B(1)
           CALL CORR(ALP,BTA,RR,NSTEP)
           WRITE(JIO,1002)NITR,NSTEP,IITIME,JITIME,F,AA,RR(1),BBB(1)
           WRITE(*,1002)NITR,NSTEP,IITIME,JITIME,F,AA,RR(1),BBB(1)
      ENDIF
C
C  ***** ROLL CALL PHASE *****
C
      IF(NSTEP.EQ.1)THEN
C
C     IF THE MIDPOINT < FURTHEST LEFT LEGISLATOR, CALL GRID ROUTINE TO
C        CALCULATE CONSTRAINED ESTIMATES OF MIDPOINT AND YES OUTCOME
C        FOR ROLL CALL
C
           IF(B(2).LT.SSS(1))call GRID2(NOBS,NEQ,KIO,B,3)
C
C     IF THE MIDPOINT > FURTHEST RIGHT LEGISLATOR, CALL GRID ROUTINE TO
C        CALCULATE CONSTRAINED ESTIMATES OF MIDPOINT AND YES OUTCOME
C        FOR ROLL CALL
C
           IF(B(2).GT.SSS(2))call GRID2(NOBS,NEQ,KIO,B,2)
C
C     IF THE YES/NO OUTCOME IS < FURTHEST LEFT LEGISLATOR AND YES/NO
C         OUTCOME IS > FURTHEST RIGHT LEGISLATOR THEN CALL GRID ROUTINE
C         TO CONSTRAIN THE YES/NO OUTCOMES BUT RETAIN MIDPOINT
C                  IF DYN(.) > 0
C
           IF((B(2)-B(1)).LT.SSS(1).AND.(B(2)+B(1)).GT.SSS(2))
     C                         call GRID2(NOBS,NEQ,KIO,B,1)
C
C                  IF DYN(.) < 0
C
           IF((B(2)+B(1)).LT.SSS(1).AND.(B(2)-B(1)).GT.SSS(2))
     C                         call GRID2(NOBS,NEQ,KIO,B,1)
C
C           WRITE(KIO,1010)NEQ,SVLNL,FNLNL,ITT,NVAL,ICNVG,
C     C     DYN(NEQ),B(1),SSSE(1),TTTE(1),V(2,1),ZMID(NEQ),B(2),
C     C     SSSE(2),TTTE(2)
C
           DYN(NEQ)=B(1)
           ZMID(NEQ)=B(2)
           STDDVZ(NEQ,1)=SSSE(1)
           STDDVZ(NEQ,2)=SSSE(2)
           ZSAVE(NEQ,2,1)=ZSAVE(NEQ,1,1)
           ZSAVE(NEQ,1,1)=B(1)
           CSAVE(NEQ,2)=CSAVE(NEQ,1)
           CSAVE(NEQ,1)=B(2)
           ZSAVE(NEQ,2,2)=FNLNL
           ZSAVE(NEQ,1,2)=SVLNL
      ENDIF
C
C  ***** LEGISLATOR PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           XDATA(NEQ)=B(1)
           XSAVE(NEQ,2,1)=XSAVE(NEQ,1,1)
           XSAVE(NEQ,1,1)=B(1)
           XSAVE(NEQ,2,2)=FNLNL
           XSAVE(NEQ,1,2)=SVLNL
           STDDVX(NEQ)=SSSE(1)
      ENDIF
      RETURN
      END
C
C  ***************************************************************************
C     SUBROUTINE CROSS---CALLED BY MAXLIK AND MAIN.  CALCULATES CROSS 
C        CLASSIFICATIONS USING PREDICTED YES AND NO PROBABILITIES VERSUS
C        ACTUAL CHOICES.
C  ***************************************************************************
C
      SUBROUTINE cross(II,LL)
      DIMENSION L(2),LL(2,2)
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      COMMON /MINE/ NS,NQ,NRCALL,NSTEP,NEQ,ISENS(513),
     CJAN,JANLST,JIO,KIO,NOBS,NOPAR,IFDRV,NFEVAL,
     CMAXIT,MAXSQZ
      COMMON /BOOK/ BBB(50)
      COMMON /DELTA/ B(50),TESTB(50),DELTAB(50),GRAD(50),V(25,25)
      BETA=BBB(1)
      WEIGHT=BBB(2)
      WTSQ=WEIGHT*WEIGHT/2.0
      DO 5 I=1,2
      L(I)=0
      DO 5 J=1,2
  5   LL(I,J)=0
C
C  ***** UTILITY FUNCTION PHASE *****
C
      IF(NSTEP.EQ.0)THEN
           DO 100 I=1,NOBS
           ICH=LDATA(I,II)
           IF(ICH.LE.0)GO TO 100
           L(ICH)=L(ICH)+1
           SUMY=(XDATA(I)-ZMID(II)+DYN(II))**2
           SUMN=(XDATA(I)-ZMID(II)-DYN(II))**2
           SUMY=BETA*EXP(-SUMY*WTSQ)
           SUMN=BETA*EXP(-SUMN*WTSQ)
           PIY=EXP(SUMY)/(EXP(SUMY)+EXP(SUMN))
           PIN=1.0-PIY
           IF(PIY.GE.PIN)LL(1,ICH)=LL(1,ICH)+1
           IF(PIY.LT.PIN)LL(2,ICH)=LL(2,ICH)+1
  100      CONTINUE
      ENDIF
C
C  ***** ROLL CALL PHASE *****
C
      IF(NSTEP.EQ.1)THEN
           DO 101 I=1,NOBS
           ICH=LDATA(I,II)
           IF(ICH.LE.0)GO TO 101
           L(ICH)=L(ICH)+1
           SUMY=(XDATA(I)-B(2)+B(1))**2
           SUMN=(XDATA(I)-B(2)-B(1))**2
           SUMY=BETA*EXP(-SUMY*WTSQ)
           SUMN=BETA*EXP(-SUMN*WTSQ)
           PIY=EXP(SUMY)/(EXP(SUMY)+EXP(SUMN))
           PIN=1.0-PIY
           IF(PIY.GE.PIN)LL(1,ICH)=LL(1,ICH)+1
           IF(PIY.LT.PIN)LL(2,ICH)=LL(2,ICH)+1
  101      CONTINUE
      ENDIF
C
C  ***** LEGISLATOR PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           DO 200 I=1,NRCALL
           ICH=LDATA(II,I)
           IF(ICH.LE.0)GO TO 200
C
C    USE THIS STATEMENT IF, ON THE LAST ITERATION, YOU DESIRE TO CALCULATE
C      THE CLASSIFICATIONS OF UNCONSTRAINED ROLL CALLS ONLY
C           if(abs(ZMID(i)).eq.1.0.AND.JANLST.EQ.JAN)go to 200
C
           L(ICH)=L(ICH)+1
           SUMY=0.0
           SUMN=0.0
           SUMY=SUMY+(XDATA(II)-ZMID(I)+DYN(I))**2
           SUMN=SUMN+(XDATA(II)-ZMID(I)-DYN(I))**2
           SUMY=BETA*EXP(-SUMY*WTSQ)
           SUMN=BETA*EXP(-SUMN*WTSQ)
           PIY=EXP(SUMY)/(EXP(SUMY)+EXP(SUMN))
           PIN=1.0-PIY
           IF(PIY.GE.PIN.AND.ICH.EQ.1)LL(1,1)=LL(1,1)+1
           IF(PIY.GE.PIN.AND.ICH.EQ.2)LL(1,2)=LL(1,2)+1
           IF(PIY.LT.PIN.AND.ICH.EQ.1)LL(2,1)=LL(2,1)+1
           IF(PIY.LT.PIN.AND.ICH.EQ.2)LL(2,2)=LL(2,2)+1
  200      CONTINUE
      ENDIF
      RETURN
      END
C
C  ***************************************************************************
C    SUBROUTINE OUTWRT---CALLED BY MAIN DURING THE ROLL CALL AND LEGISLATOR
C      PHASES ONLY.  CALCULATES NUMBER OF CONSTRAINED ROLL CALLS AND GMP 
C      FOR EACH ROLL CALL DURING ROLL CALL PHASE AND, AT THE LAST ITERATION,
C      WRITES ROLL CALL ESTIMATES TO DISK.  FOR LEGISLATOR PHASE, THE GMP
C      FOR EACH LEGISLATOR IS CALCULATED AND, AT THE LAST ITERATION, WRITES
C      LEGISLATOR ESTIMATES TO DISK.  FOR BOTH ROLL CALL AND LEGISLATOR 
C      PHASES, THE CPU TIME OF THE PHASE IS CALCULATED, WRITTEN OUT, AND
C      STORED.
C  ***************************************************************************
C
      SUBROUTINE OUTWRT(NRX,NSTEP,JIO,KIO,JAN,JANLST,KPJP,YBIGL,FMT2)
      DIMENSION YBIGL(1601),KPJP(1601,4),ALP(2),BTA(2),RR(2)
      character*1 kname
      CHARACTER*64 FMT2
      COMMON /XXCOM/ ZMID(1601),XDATA(513),DYN(1601),ldata(455,1600)
      COMMON /XIME/ NTIME(20),NITR,MSEC
      COMMON /VOTES/ KAV(1601),kay(1601),kan(1601),kname(513,25),jne
      COMMON /STD/ STDDVX(513),STDDVZ(1601,3)
      COMMON /BOOK/ BBB(50)
      COMMON /BAD/KBAD1,KBAD2,SPRM,SPRSD
 1002 FORMAT(1X,2I4,I2,4x,4i3,1x,7F7.3)
 1003 FORMAT(1X,2I4,2I7,F14.4,3F9.4)
 1004 FORMAT(20A4)
 1005 FORMAT(1X,I4,1X,19A1,4I4,4F10.4)
 1014 FORMAT(1X,2I7,2G20.13)
 1015 format(1x,i4,5x,5A1,4x,2A1,2x,A1,3x,11A1,1x,3f7.3,4I4)
C
C  ***** ROLL CALL PHASE *****
C
      IF(NSTEP.EQ.1)THEN
           KBAD1=0
           KBAD2=0
           SPRM=0.0
           SPRSD=0.0
           SUM=0.0
           KKSUM=0
           DO 11 I=1,NRX
           KSUM=0
C
C  KBAD1 IS THE NUMBER OF ROLL CALLS WITH MIDPOINTS AT +1.0/-1.0
C
           KVYBAD=0
           IF(ABS(ABS(ZMID(I))-1.0).LT..001)KBAD1=KBAD1+1
           IF(ABS(ABS(ZMID(I))-1.0).LT..001)KVYBAD=1
C
C  KBAD2 IS THE NUMBER OF ROLL CALLS WITH YES AND NO OUTCOMES OFF THE
C    ENDS OF THE LEGISLATOR CONFIGURATION
C
           IF(ABS(ABS(ZMID(I)-DYN(I))-1.0).LT.0.001.OR.
     C        ABS(ABS(ZMID(I)+DYN(I))-1.0).LT.0.001)KBAD2=KBAD2+1
           IF(ABS(ABS(ZMID(I)-DYN(I))-1.0).LT.0.001.OR.
     C        ABS(ABS(ZMID(I)+DYN(I))-1.0).LT.0.001)KVYBAD=2
           SPRM=SPRM+ABS(2.0*DYN(I))
           SPRSD=SPRSD+(2.0*DYN(I))**2
           DO 12 JJ=1,4
  12       KSUM=KSUM+KPJP(I,JJ)
           KKSUM=KKSUM+KSUM
           SUM=SUM+YBIGL(I)
           IF(KSUM.EQ.0)AA=99.0
           IF(KSUM.EQ.0)GO TO 16
           AA=YBIGL(I)/FLOAT(KSUM)
           AA=EXP(AA)
  16       CONTINUE
           IF(JAN.LT.JANLST)GO TO 11
           WRITE(33,1002)I,KAV(I),KVYBAD,(KPJP(I,J),J=1,4),AA,DYN(I),
     C     ZMID(I),(STDDVZ(I,J),J=1,2)
  11       CONTINUE
C
C   SPRM IS THE AVERAGE DISTANCE BETWEEN THE YES AND NO OUTCOMES AND
C      SPRSD IS THE STANDARD DEVIATION OF THE DISTANCE
C
           SPRSD=SQRT(FLOAT(NRX)*SPRSD-SPRM*SPRM)/
     C     FLOAT(NRX)
           SPRM=SPRM/FLOAT(NRX)
      ENDIF
C
C ***** LEGISLATOR PHASE *****
C
      IF(NSTEP.EQ.2)THEN
           SUM=0.0
           KKSUM=0
           DO 13 I=1,NRX
           KSUM=0
           DO 14 JJ=1,4
  14       KSUM=KSUM+KPJP(I,JJ)
           KKSUM=KKSUM+KSUM
           SUM=SUM+YBIGL(I)
           IF(KSUM.EQ.0)AA=99.0
           IF(KSUM.EQ.0)GO TO 166
           AA=YBIGL(I)/FLOAT(KSUM)
           AA=EXP(AA)
 166       CONTINUE
           IF(JAN.LT.JANLST)GO TO 13
C
C  WRITE OUT LEGISLATOR COORDINATES--NOTE THAT THIS FORMAT IS UNIQUE
C    TO THE ROLL CALL DATA -- FOR DIFFERENT MATRICES, THIS CODE
C    WILL HAVE TO BE COMMENTED OUT -- SEE EXAMPLE BELOW
C
           WRITE(31,FMT2)I,(KNAME(I,J),J=4,8),kname(i,1),
     C     kname(i,2),kname(i,3),(kname(i,j),j=9,JNE),AA,xdata(i),
     C     STDDVX(I),(KPJP(I,J),J=1,4)
C           WRITE(31,FMT2)I,(KNAME(I,J),J=1,JNE),AA,XDATA(I),
C     C     STDDVX(I),(KPJP(I,J),J=1,4)
  13       CONTINUE
      ENDIF
C
C  CALCULATE CPU TIME OF CURRENT PHASE
C
      call CPUTIM(KTIM1)
      IITIME=KTIM1
      IF(NITR.EQ.1)JITIME=IITIME-NTIME(1)
      NTIME(NITR)=IITIME
      IF(NITR.GT.1)JITIME=NTIME(NITR)-NTIME(NITR-1)
      AA=SUM/FLOAT(KKSUM)
      AA=EXP(AA)
      CALL CORR(ALP,BTA,RR,NSTEP)
      WRITE(KIO,1014)NRX,KKSUM,SUM,AA
      IF(NSTEP.EQ.2)THEN
           WRITE(JIO,1003)NITR,NSTEP,IITIME,JITIME,SUM,AA,RR(1)
           IF(JIO.NE.KIO)WRITE(*,1003)NITR,NSTEP,IITIME,JITIME,SUM,AA,
     C                                RR(1)
      ENDIF
      IF(NSTEP.EQ.1)THEN
           WRITE(JIO,1003)NITR,NSTEP,IITIME,JITIME,SUM,AA,RR(1),RR(2)
           IF(JIO.NE.KIO)WRITE(*,1003)NITR,NSTEP,IITIME,JITIME,SUM,AA,
     C                                RR(1),RR(2)
      ENDIF
      RETURN
      END
